#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import MO
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, APR, MAY, JUN, JUL, AUG, SEP, OCT, DEC
from holidays.constants import MON, TUE, WED, THU, FRI, SAT, SUN
from holidays.holiday_base import HolidayBase
from holidays.utils import _ChineseLuniSolar


class HongKong(HolidayBase):
    """
    https://en.wikipedia.org/wiki/Public_holidays_in_Hong_Kong
    Holidays for 2007–2023 (government source):
    https://www.gov.hk/en/about/abouthk/holiday/index.htm
    """

    country = "HK"
    special_holidays = {
        # Special holiday on 2015 - The 70th anniversary day of the victory
        # of the Chinese people's war of resistance against Japanese
        # aggression.
        2015: (
            (
                SEP,
                3,
                "The 70th anniversary day of the victory of the Chinese "
                "people's war of resistance against Japanese aggression",
            ),
        ),
    }

    def __init__(self, **kwargs):
        self.cnls = _ChineseLuniSolar()
        HolidayBase.__init__(self, **kwargs)

    def _populate(self, year):
        # Current set of holidays actually valid since 1946
        if year <= 1945:
            return
        super()._populate(year)

        day_following = "The day following "

        # The first day of January
        first_date = date(year, JAN, 1)
        if self.observed and first_date.weekday() == SUN:
            self[first_date + rd(days=+1)] = (
                day_following + "the first day of January"
            )
        else:
            self[first_date] = "The first day of January"

        # Lunar New Year
        name = "Lunar New Year's Day"
        preceding_day_lunar = "The day preceding Lunar New Year's Day"
        second_day_lunar = "The second day of Lunar New Year"
        third_day_lunar = "The third day of Lunar New Year"
        fourth_day_lunar = "The fourth day of Lunar New Year"
        new_year_date = self.cnls.lunar_n_y_date(year)
        if self.observed:
            self[new_year_date] = name
            if new_year_date.weekday() in {MON, TUE, WED, THU}:
                self[new_year_date] = name
                self[new_year_date + rd(days=+1)] = second_day_lunar
                self[new_year_date + rd(days=+2)] = third_day_lunar
            if new_year_date.weekday() == FRI:
                self[new_year_date] = name
                self[new_year_date + rd(days=+1)] = second_day_lunar
                self[new_year_date + rd(days=+3)] = fourth_day_lunar
            if new_year_date.weekday() == SAT:
                self[new_year_date] = name
                self[new_year_date + rd(days=+2)] = third_day_lunar
                self[new_year_date + rd(days=+3)] = fourth_day_lunar
            if new_year_date.weekday() == SUN:
                if year in {2006, 2007, 2010}:
                    self[new_year_date + rd(days=-1)] = preceding_day_lunar
                else:
                    self[new_year_date + rd(days=+3)] = fourth_day_lunar
            else:
                self[new_year_date] = name
            if new_year_date.weekday() == SAT:
                self[new_year_date + rd(days=+3)] = fourth_day_lunar
            else:
                self[new_year_date + rd(days=+1)] = second_day_lunar
            if new_year_date.weekday() == FRI:
                self[new_year_date + rd(days=+3)] = fourth_day_lunar
            else:
                self[new_year_date + rd(days=+2)] = third_day_lunar
        else:
            self[new_year_date] = name
            self[new_year_date + rd(days=+1)] = second_day_lunar
            self[new_year_date + rd(days=+2)] = third_day_lunar

        easter_date = easter(year)
        easter_monday_date = easter_date + rd(days=+1)
        # Ching Ming Festival
        name = "Ching Ming Festival"
        if self.is_leap_year(year) or (
            year > 2008 and self.is_leap_year(year - 1)
        ):
            ching_ming_date = date(year, APR, 4)
        else:
            ching_ming_date = date(year, APR, 5)
        if self.observed and (
            ching_ming_date.weekday() == SUN
            or ching_ming_date == easter_monday_date
        ):
            self[ching_ming_date + rd(days=+1)] = day_following + name
        else:
            self[ching_ming_date] = name

        # Easter Holiday
        good_friday = "Good Friday"
        easter_monday = "Easter Monday"
        self[easter_date + rd(days=-2)] = good_friday
        self[easter_date + rd(days=-1)] = day_following + good_friday
        if self.observed and self.get(easter_monday_date):
            self[easter_monday_date + rd(days=+1)] = (
                day_following + easter_monday
            )
        else:
            self[easter_monday_date] = easter_monday

        # The Birthday of the Buddha
        if year >= 1998:
            name = "The Birthday of the Buddha"
            buddha_date = self.cnls.lunar_to_gre(year, 4, 8)
            if self.observed and buddha_date.weekday() == SUN:
                self[buddha_date + rd(days=+1)] = day_following + name
            else:
                self[buddha_date] = name

        # Labour Day
        if year >= 1998:
            name = "Labour Day"
            labour_date = date(year, MAY, 1)
            if self.observed and labour_date.weekday() == SUN:
                self[labour_date + rd(days=+1)] = day_following + name
            else:
                self[labour_date] = name

        # Tuen Ng Festival
        name = "Tuen Ng Festival"
        tuen_ng_date = self.cnls.lunar_to_gre(year, 5, 5)
        if self.observed and tuen_ng_date.weekday() == SUN:
            self[tuen_ng_date + rd(days=+1)] = day_following + name
        else:
            self[tuen_ng_date] = name

        # Hong Kong Special Administrative Region Establishment Day
        if year >= 1997:
            name = "Hong Kong Special Administrative Region Establishment Day"
            hksar_date = date(year, JUL, 1)
            if self.observed and hksar_date.weekday() == SUN:
                self[hksar_date + rd(days=+1)] = day_following + name
            else:
                self[hksar_date] = name

        # Chinese Mid-Autumn Festival
        name = "Chinese Mid-Autumn Festival"
        mid_autumn_date = self.cnls.lunar_to_gre(year, 8, 15)
        if self.observed:
            # if Chinese Mid-Autumn Festival lies on Saturday
            # before 1983 public holiday lies on Monday
            # from 1983 to 2010 public holiday lies on same day
            # since 2011 public holiday lies on Monday
            if mid_autumn_date.weekday() == SAT:
                if 1983 <= year <= 2010:
                    self[mid_autumn_date] = name
                else:
                    self[mid_autumn_date + rd(days=+2)] = (
                        "The second day of the " + name + " (Monday)"
                    )
            else:
                self[mid_autumn_date + rd(days=+1)] = (
                    day_following + "the " + name
                )
        else:
            self[mid_autumn_date] = name

        # National Day
        if year >= 1997:
            name = "National Day"
            national_date = date(year, OCT, 1)
            if self.observed and (
                national_date.weekday() == SUN or self.get(national_date)
            ):
                self[national_date + rd(days=+1)] = day_following + name
            else:
                self[national_date] = name

        # Chung Yeung Festival
        name = "Chung Yeung Festival"
        chung_yeung_date = self.cnls.lunar_to_gre(year, 9, 9)
        if self.observed and chung_yeung_date.weekday() == SUN:
            self[chung_yeung_date + rd(days=+1)] = day_following + name
        else:
            self[chung_yeung_date] = name

        # Christmas Day
        name = "Christmas Day"
        first_after_christmas = "The first weekday after " + name
        second_after_christmas = "The second weekday after " + name
        christmas_date = date(year, DEC, 25)
        if self.observed:
            if christmas_date.weekday() == SUN:
                self[christmas_date + rd(days=+1)] = first_after_christmas
                self[christmas_date + rd(days=+2)] = second_after_christmas
            elif christmas_date.weekday() == SAT:
                self[christmas_date] = name
                self[christmas_date + rd(days=+2)] = first_after_christmas
            else:
                self[christmas_date] = name
                self[christmas_date + rd(days=+1)] = first_after_christmas
        else:
            self[christmas_date] = name
            self[christmas_date + rd(days=+1)] = day_following + name

        # Previous holidays
        if 1952 <= year <= 1997:
            # Queen's Birthday (June 2nd Monday)
            dt = date(year, JUN, 1) + rd(weekday=MO(+2))
            self[dt] = "Queen's Birthday"

        if year <= 1996:
            # Anniversary of the liberation of Hong Kong (August last Monday)
            dt = date(year, AUG, 31) + rd(weekday=MO(-1))
            self[dt] = "Anniversary of the liberation of Hong Kong"

            # Anniversary of the victory in the Second Sino-Japanese War
            self[
                dt + rd(days=-1)
            ] = "Anniversary of the victory in the Second Sino-Japanese War"

    @staticmethod
    def is_leap_year(year):
        return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)


class HK(HongKong):
    pass


class HKG(HongKong):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAR, MAY, JUL, AUG, NOV
from holidays.holiday_base import HolidayBase
from holidays.utils import _islamic_to_gre


class Morocco(HolidayBase):
    """
    Moroccan holidays
    Note that holidays falling on a sunday is "lost",
    it will not be moved to another day to make up for the collision.

    # Holidays after 2020: the following four moving date holidays whose exact
    # date is announced yearly are estimated (and so denoted):
    # - Eid El Fetr*
    # - Eid El Adha*
    # - 1er Moharram*
    # - Aid al Mawlid Annabawi*
    # *only if hijri-converter library is installed, otherwise a warning is
    #  raised that this holiday is missing. hijri-converter requires
    #  Python >= 3.6
    Primary sources:
    https://fr.wikipedia.org/wiki/F%C3%AAtes_et_jours_f%C3%A9ri%C3%A9s_au_Maroc
    https://www.mmsp.gov.ma/fr/pratiques.aspx?id=38
    """

    country = "MA"

    def _populate(self, year):
        super()._populate(year)

        def _add_holiday(dt: date, hol: str) -> None:
            """Only add if in current year; prevents adding holidays across
            years (handles multi-day Islamic holidays that straddle Gregorian
            years).
            """
            if dt.year == year:
                self[dt] = hol

        # New Year's Day
        self[date(year, JAN, 1)] = "Nouvel an - Premier janvier"

        # Independence Manifesto Day post 1944
        if year > 1944:
            self[date(year, JAN, 11)] = (
                "Commémoration de la présentation "
                "du manifeste de l'indépendance"
            )

        # Labor day
        self[date(year, MAY, 1)] = "Fête du Travail"

        # Throne day
        if year > 2000:
            self[date(year, JUL, 30)] = "Fête du Trône"
        elif year > 1962:
            self[date(year, MAR, 3)] = "Fête du Trône"
        else:
            self[date(year, NOV, 18)] = "Fête du Trône"

        # Oued Ed-Dahab Day
        self[date(year, AUG, 14)] = "Journée de Oued Ed-Dahab"

        # Revolution Day
        self[date(year, AUG, 20)] = (
            "Commémoration de la révolution du " "Roi et du peuple"
        )

        # Youth day
        if year > 2000:
            self[date(year, AUG, 21)] = "Fête de la jeunesse"
        else:
            self[date(year, JUL, 9)] = "Fête du Trône"

        # Green March
        if year > 1975:
            self[date(year, NOV, 6)] = "Marche verte"

        # Independance day
        if year > 1956:
            self[date(year, NOV, 18)] = "Fête de l'indépendance"

        # Eid al-Fitr - Feast Festive
        # date of observance is announced yearly, This is an estimate since
        # having the Holiday on Weekend does change the number of days,
        # deceided to leave it since marking a Weekend as a holiday
        # wouldn't do much harm.
        for yr in (year - 1, year):
            for date_obs in _islamic_to_gre(yr, 10, 1):
                hol_date = date_obs
                _add_holiday(hol_date, "Eid al-Fitr")
                _add_holiday(hol_date + rd(days=1), "Eid al-Fitr")

        # Eid al-Adha - Sacrifice Festive
        # date of observance is announced yearly
        for yr in (year - 1, year):
            for date_obs in _islamic_to_gre(yr, 12, 10):
                hol_date = date_obs
                _add_holiday(hol_date, "Eid al-Adha")
                _add_holiday(hol_date + rd(days=1), "Eid al-Adha")

        # Islamic New Year - (hijari_year, 1, 1)
        for date_obs in _islamic_to_gre(year, 1, 1):
            hol_date = date_obs
            self[hol_date] = "1er Moharram"

        # Prophet Muhammad's Birthday - (hijari_year, 3, 12)
        for yr in (year - 1, year):
            for date_obs in _islamic_to_gre(yr, 3, 12):
                hol_date = date_obs
                _add_holiday(hol_date, "Aid al Mawlid Annabawi")
                _add_holiday(hol_date + rd(days=1), "Aid al Mawlid Annabawi")


class MA(Morocco):
    pass


class MOR(Morocco):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.holiday_base import HolidayBase


class Latvia(HolidayBase):
    """
    https://en.wikipedia.org/wiki/Public_holidays_in_Latvia
    https://information.lv/
    """

    country = "LV"

    def _populate(self, year):
        super()._populate(year)

        # New Year's Day
        self[date(year, 1, 1)] = "Jaunais gads"

        # Good Friday
        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Lielā Piektdiena"

        # Easter
        self[easter_date] = "Lieldienas"

        # Easter 2nd day
        self[easter_date + rd(days=+1)] = "Otrās Lieldienas"

        # International Workers' Day
        self[date(year, 5, 1)] = "Darba svētki"

        # Restoration of Independence Day (1990).
        # Latvia proclaimed its independence from the USSR,
        #   and restoration of the Republic of Latvia.
        if year >= 1990:
            self[
                date(year, 5, 4)
            ] = "Latvijas Republikas Neatkarības atjaunošanas diena"

        # by law
        # https://likumi.lv/ta/id/72608-par-svetku-atceres-un-atzimejamam-dienam
        # Midsummer's Eve
        if year >= 1990:
            self[date(year, 6, 23)] = "Līgo diena"

        # Midsummer's Day
        # Also so called "St. John's Day"
        if year >= 1990:
            self[date(year, 6, 24)] = "Jāņu dienu"

        # Proclamation Day of the Republic of Latvia
        if year >= 1918:
            self[
                date(year, 11, 18)
            ] = "Latvijas Republikas proklamēšanas diena"

        # Christmas Eve
        self[date(year, 12, 24)] = "Ziemassvētku vakars"

        # Christmas 1st day
        self[date(year, 12, 25)] = "Ziemassvētki"

        # Christmas 2nd day
        self[date(year, 12, 26)] = "Otrie Ziemassvētki"

        # New Year's Eve
        self[date(year, 12, 31)] = "Vecgada vakars"


class LV(Latvia):
    pass


class LVA(Latvia):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.relativedelta import MO
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import FRI, SAT, SUN, JAN, FEB, MAR, MAY, SEP, NOV, DEC
from holidays.holiday_base import HolidayBase


class Mexico(HolidayBase):

    country = "MX"

    def _add_with_observed(self, holiday: date, name: str):
        self[holiday] = name
        if self.observed and holiday.weekday() == SAT:
            self[holiday + rd(days=-1)] = name + " (Observed)"
        elif self.observed and holiday.weekday() == SUN:
            self[holiday + rd(days=+1)] = name + " (Observed)"

    def _populate(self, year):
        super()._populate(year)

        # New Year's Day
        name = "Año Nuevo [New Year's Day]"
        dt = date(year, JAN, 1)
        self[dt] = name
        if self.observed and dt.weekday() == SUN:
            self[dt + rd(days=+1)] = name + " (Observed)"
        # The next year's observed New Year's Day can be in this year
        # when it falls on a Friday (Jan 1st is a Saturday)
        if self.observed and date(year, DEC, 31).weekday() == FRI:
            self[date(year, DEC, 31)] = name + " (Observed)"

        # Constitution Day
        name = "Día de la Constitución [Constitution Day]"
        if self.observed and year >= 2007:
            self[date(year, FEB, 1) + rd(weekday=MO(+1))] = (
                name + " (Observed)"
            )

        if year >= 1917:
            self[date(year, FEB, 5)] = name

        # Benito Juárez's birthday
        name = "Natalicio de Benito Juárez [Benito Juárez's birthday]"
        if self.observed and year >= 2007:
            self[date(year, MAR, 1) + rd(weekday=MO(+3))] = (
                name + " (Observed)"
            )

        if year >= 1917:
            self[date(year, MAR, 21)] = name

        # Labor Day
        if year >= 1923:
            name = "Día del Trabajo [Labour Day]"
            dt = date(year, MAY, 1)
            self._add_with_observed(dt, name)

        # Independence Day
        name = "Día de la Independencia [Independence Day]"
        dt = date(year, SEP, 16)
        self._add_with_observed(dt, name)

        # Revolution Day
        name = "Día de la Revolución [Revolution Day]"
        if self.observed and year >= 2007:
            self[date(year, NOV, 1) + rd(weekday=MO(+3))] = (
                name + " (Observed)"
            )

        if year >= 1917:
            self[date(year, NOV, 20)] = name

        # Change of Federal Government
        # Every six years--next observance 2018
        if year >= 1970 and (2096 - year) % 6 == 0:
            name = (
                "Transmisión del Poder Ejecutivo Federal"
                " [Change of Federal Government]"
            )
            dt = date(year, DEC, 1)
            self._add_with_observed(dt, name)

        # Christmas
        name = "Navidad [Christmas]"
        dt = date(year, DEC, 25)
        self._add_with_observed(dt, name)


class MX(Mexico):
    pass


class MEX(Mexico):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAY, JUN, AUG, NOV, DEC
from holidays.holiday_base import HolidayBase


class Luxembourg(HolidayBase):
    """
    https://en.wikipedia.org/wiki/Public_holidays_in_Luxembourg
    """

    country = "LU"

    def _populate(self, year):
        super()._populate(year)

        # Public holidays
        self[date(year, JAN, 1)] = "Neijoerschdag"
        easter_date = easter(year)
        self[easter_date + rd(days=+1)] = "Ouschterméindeg"
        self[date(year, MAY, 1)] = "Dag vun der Aarbecht"
        if year >= 2019:
            # Europe Day: not in legislation yet, but introduced starting 2019
            self[date(year, MAY, 9)] = "Europadag"
        self[easter_date + rd(days=+39)] = "Christi Himmelfaart"
        self[easter_date + rd(days=+50)] = "Péngschtméindeg"
        self[date(year, JUN, 23)] = "Nationalfeierdag"
        self[date(year, AUG, 15)] = "Léiffrawëschdag"
        self[date(year, NOV, 1)] = "Allerhellgen"
        self[date(year, DEC, 25)] = "Chrëschtdag"
        self[date(year, DEC, 26)] = "Stiefesdag"


class LU(Luxembourg):
    pass


class LUX(Luxembourg):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from .angola import AO, AGO, Angola
from .argentina import AR, ARG, Argentina
from .armenia import AM, ARM, Armenia
from .aruba import AW, ABW, Aruba
from .australia import AU, AUS, Australia
from .austria import AT, AUT, Austria
from .azerbaijan import AZ, AZE, Azerbaijan
from .bangladesh import BD, BGD, Bangladesh
from .belarus import BY, BLR, Belarus
from .belgium import BE, BEL, Belgium
from .bolivia import BO, BOL, Bolivia
from .bosnia_and_herzegovina import BA, BIH, BosniaAndHerzegovina
from .botswana import BW, BWA, Botswana
from .brazil import BR, BRA, Brazil
from .bulgaria import BG, BLG, Bulgaria
from .burundi import BI, BDI, Burundi
from .canada import CA, CAN, Canada
from .chile import CL, CHL, Chile
from .china import CN, CHN, China
from .colombia import CO, COL, Colombia
from .croatia import HR, HRV, Croatia
from .cuba import CU, CUB, Cuba
from .curacao import CW, CUW, Curacao
from .cyprus import CY, CYP, Cyprus
from .czechia import CZ, CZE, Czechia
from .denmark import DK, DNK, Denmark
from .djibouti import DJ, DJI, Djibouti
from .dominican_republic import DO, DOM, DominicanRepublic
from .egypt import EG, EGY, Egypt
from .estonia import EE, EST, Estonia
from .eswatini import SZ, SZW, Eswatini, Swaziland
from .ethiopia import ET, ETH, Ethiopia
from .finland import FI, FIN, Finland
from .france import FR, FRA, France
from .georgia import GE, GEO, Georgia
from .germany import DE, DEU, Germany
from .greece import GR, GRC, Greece
from .honduras import HN, HND, Honduras
from .hongkong import HK, HKG, HongKong
from .hungary import HU, HUN, Hungary
from .iceland import IS, ISL, Iceland
from .india import IN, IND, India
from .indonesia import ID, IDN, Indonesia
from .ireland import IE, IRL, Ireland
from .isle_of_man import IM, IMN, IsleOfMan
from .israel import IL, ISR, Israel
from .italy import IT, ITA, Italy
from .jamaica import JM, JAM, Jamaica
from .japan import JP, JPN, Japan
from .kazakhstan import KZ, KAZ, Kazakhstan
from .kenya import KE, KEN, Kenya
from .latvia import LV, LVA, Latvia
from .lesotho import LS, LSO, Lesotho
from .liechtenstein import LI, LIE, Liechtenstein
from .lithuania import LT, LTU, Lithuania
from .luxembourg import LU, LUX, Luxembourg
from .madagascar import MG, MDG, Madagascar
from .malawi import MW, MWI, Malawi
from .malaysia import MY, MYS, Malaysia
from .malta import MT, MLT, Malta
from .mexico import MX, MEX, Mexico
from .moldova import MD, MDA, Moldova
from .morocco import MA, MOR, Morocco
from .mozambique import MZ, MOZ, Mozambique
from .namibia import NA, NAM, Namibia
from .netherlands import NL, NLD, Netherlands
from .new_zealand import NZ, NZL, NewZealand
from .nicaragua import NI, NIC, Nicaragua
from .nigeria import NG, NGA, Nigeria
from .north_macedonia import MK, MKD, NorthMacedonia
from .norway import NO, NOR, Norway
from .pakistan import PK, PAK, Pakistan
from .paraguay import PY, PRY, Paraguay
from .peru import PE, PER, Peru
from .poland import PL, POL, Poland
from .portugal import PT, PRT, Portugal
from .romania import RO, ROU, Romania
from .russia import RU, RUS, Russia
from .saudi_arabia import SA, SAU, SaudiArabia
from .serbia import RS, SRB, Serbia
from .singapore import SG, SGP, Singapore
from .slovakia import SK, SVK, Slovakia
from .slovenia import SI, SVN, Slovenia
from .south_africa import ZA, ZAF, SouthAfrica
from .south_korea import KR, KOR, Korea, SouthKorea
from .spain import ES, ESP, Spain
from .sweden import SE, SWE, Sweden
from .switzerland import CH, CHE, Switzerland
from .taiwan import TW, TWN, Taiwan
from .tunisia import TN, TUN, Tunisia
from .turkey import TR, TUR, Turkey
from .ukraine import UA, UKR, Ukraine
from .united_arab_emirates import AE, ARE, UnitedArabEmirates
from .united_kingdom import GB, UK, GBR, UnitedKingdom
from .united_states import US, USA, UnitedStates
from .uruguay import UY, URY, Uruguay
from .uzbekistan import UZ, UZB, Uzbekistan
from .venezuela import VE, VEN, Venezuela
from .vietnam import VN, VNM, Vietnam
from .zambia import ZM, ZMB, Zambia
from .zimbabwe import ZW, ZWE, Zimbabwe



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)
from datetime import date

from dateutil.relativedelta import FR
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JUN, JUL

from .united_kingdom import UnitedKingdom


class IsleOfMan(UnitedKingdom):
    """Using existing code in UnitedKingdom for now."""

    country = "IM"

    def _populate(self, year: int) -> None:
        super()._populate(year)

        # Isle of Man exclusive holidays
        # TT bank holiday (first Friday in June)
        self[date(year, JUN, 1) + rd(weekday=FR)] = "TT Bank Holiday"

        # Tynwald Day
        self[date(year, JUL, 5)] = "Tynwald Day"


class IM(IsleOfMan):
    pass


class IMN(IsleOfMan):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd
from dateutil.relativedelta import MO

from holidays.constants import JAN, MAR, MAY, JUL, OCT, DEC
from holidays.holiday_base import HolidayBase


class Malawi(HolidayBase):
    """
    https://www.officeholidays.com/countries/malawi
    https://www.timeanddate.com/holidays/malawi/
    """

    country = "MW"

    def _populate(self, year):
        # Observed since 2000
        if year <= 1999:
            return
        super()._populate(year)

        self[date(year, JAN, 1)] = "New Year's Day"

        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Good Friday"
        self[easter_date + rd(days=+1)] = "Easter Monday"

        self[date(year, JAN, 15)] = "John Chilembwe Day"
        self[date(year, MAR, 3)] = "Martyrs Day"
        self[date(year, MAY, 1)] = "Labour Day"
        self[date(year, MAY, 14)] = "Kamuzu Day"
        self[date(year, JUL, 6)] = "Independence Day"
        self[date(year, OCT, 15)] = "Mother's Day"
        self[date(year, DEC, 25)] = "Christmas Day"
        self[date(year, DEC, 26)] = "Boxing Day"

        if self.observed:
            for k, v in list(self.items()):
                if self._is_weekend(k) and k.year == year:
                    self[k + rd(weekday=MO)] = v + " (Observed)"


class MW(Malawi):
    pass


class MWI(Malawi):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAY, JUN, JUL, AUG, OCT, NOV, DEC
from holidays.holiday_base import HolidayBase


class Peru(HolidayBase):
    """
    https://www.gob.pe/feriados
    https://es.wikipedia.org/wiki/Anexo:Días_feriados_en_el_Perú
    """

    country = "PE"

    def _populate(self, year):
        super()._populate(year)

        # New Year's Day
        self[date(year, JAN, 1)] = "Año Nuevo [New Year's Day]"

        # Feast of Saints Peter and Paul
        name = "San Pedro y San Pablo [Feast of Saints Peter and Paul]"
        self[date(year, JUN, 29)] = name

        # Independence Day
        name = "Día de la Independencia [Independence Day]"
        self[date(year, JUL, 28)] = name

        name = "Día de las Fuerzas Armadas y la Policía del Perú"
        self[date(year, JUL, 29)] = name

        # Santa Rosa de Lima
        name = "Día de Santa Rosa de Lima"
        self[date(year, AUG, 30)] = name

        # Battle of Angamos
        name = "Combate Naval de Angamos [Battle of Angamos]"
        self[date(year, OCT, 8)] = name

        easter_date = easter(year)
        # Holy Thursday
        self[easter_date + rd(days=-3)] = "Jueves Santo [Maundy Thursday]"

        # Good Friday
        self[easter_date + rd(days=-2)] = "Viernes Santo [Good Friday]"

        # Holy Saturday
        self[easter_date + rd(days=-1)] = "Sábado de Gloria [Holy Saturday]"

        # Easter Sunday
        self[easter_date] = "Domingo de Resurrección [Easter Sunday]"

        # Labor Day
        self[date(year, MAY, 1)] = "Día del Trabajo [Labour Day]"

        # All Saints Day
        name = "Día de Todos Los Santos [All Saints Day]"
        self[date(year, NOV, 1)] = name

        # Inmaculada Concepción
        name = "Inmaculada Concepción [Immaculate Conception]"
        self[date(year, DEC, 8)] = name

        # Christmas
        self[date(year, DEC, 25)] = "Navidad [Christmas]"


class PE(Peru):
    pass


class PER(Peru):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import TH, SU
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAR, APR, MAY, JUN, AUG, SEP, NOV, DEC
from holidays.holiday_base import HolidayBase


class Switzerland(HolidayBase):
    country = "CH"
    subdivisions = [
        "AG",
        "AR",
        "AI",
        "BL",
        "BS",
        "BE",
        "FR",
        "GE",
        "GL",
        "GR",
        "JU",
        "LU",
        "NE",
        "NW",
        "OW",
        "SG",
        "SH",
        "SZ",
        "SO",
        "TG",
        "TI",
        "UR",
        "VD",
        "VS",
        "ZG",
        "ZH",
    ]

    def _populate(self, year):
        super()._populate(year)

        # public holidays
        self[date(year, JAN, 1)] = "Neujahrestag"

        if self.subdiv in {
            "AG",
            "BE",
            "FR",
            "GL",
            "GR",
            "JU",
            "LU",
            "NE",
            "OW",
            "SH",
            "SO",
            "TG",
            "VD",
            "ZG",
            "ZH",
        }:
            self[date(year, JAN, 2)] = "Berchtoldstag"

        if self.subdiv in {"SZ", "TI", "UR"}:
            self[date(year, JAN, 6)] = "Heilige Drei Könige"

        if self.subdiv == "NE":
            self[date(year, MAR, 1)] = "Jahrestag der Ausrufung der Republik"

        if self.subdiv in {"NW", "SZ", "TI", "UR", "VS"}:
            self[date(year, MAR, 19)] = "Josefstag"

        easter_date = easter(year)
        # Näfelser Fahrt (first Thursday in April but not in Holy Week)
        if self.subdiv == "GL" and year >= 1835:
            dt = date(year, APR, 1) + rd(weekday=TH)
            if dt == easter_date + rd(days=-3):
                dt += rd(days=+7)
            self[dt] = "Näfelser Fahrt"

        # it's a Holiday on a Sunday
        self[easter_date] = "Ostern"

        # VS don't have easter
        if self.subdiv != "VS":
            self[easter_date + rd(days=-2)] = "Karfreitag"
            self[easter_date + rd(days=+1)] = "Ostermontag"

        if self.subdiv in {
            "BL",
            "BS",
            "JU",
            "NE",
            "SH",
            "SO",
            "TG",
            "TI",
            "ZH",
        }:
            self[date(year, MAY, 1)] = "Tag der Arbeit"

        self[easter_date + rd(days=+39)] = "Auffahrt"

        # it's a Holiday on a Sunday
        self[easter_date + rd(days=+49)] = "Pfingsten"

        self[easter_date + rd(days=+50)] = "Pfingstmontag"

        if self.subdiv in {
            "AI",
            "JU",
            "LU",
            "NW",
            "OW",
            "SZ",
            "TI",
            "UR",
            "VS",
            "ZG",
        }:
            self[easter_date + rd(days=+60)] = "Fronleichnam"

        if self.subdiv == "JU":
            self[date(year, JUN, 23)] = "Fest der Unabhängigkeit"

        if self.subdiv == "TI":
            self[date(year, JUN, 29)] = "Peter und Paul"

        if year >= 1291:
            self[date(year, AUG, 1)] = "Nationalfeiertag"

        if self.subdiv in {
            "AI",
            "JU",
            "LU",
            "NW",
            "OW",
            "SZ",
            "TI",
            "UR",
            "VS",
            "ZG",
        }:
            self[date(year, AUG, 15)] = "Mariä Himmelfahrt"

        if self.subdiv == "VD":
            # Monday after the third Sunday of September
            dt = date(year, SEP, 1) + rd(weekday=SU(+3)) + rd(days=+1)
            self[dt] = "Lundi du Jeûne"

        if self.subdiv == "GE":
            # Thursday after the first Sunday of September
            dt = date(year, SEP, 1) + rd(weekday=SU) + rd(days=+4)
            self[dt] = "Jeûne genevois"

        if self.subdiv == "OW":
            self[date(year, SEP, 25)] = "Bruder Klaus"

        if self.subdiv in {
            "AI",
            "GL",
            "JU",
            "LU",
            "NW",
            "OW",
            "SG",
            "SZ",
            "TI",
            "UR",
            "VS",
            "ZG",
        }:
            self[date(year, NOV, 1)] = "Allerheiligen"

        if self.subdiv in {
            "AI",
            "LU",
            "NW",
            "OW",
            "SZ",
            "TI",
            "UR",
            "VS",
            "ZG",
        }:
            self[date(year, DEC, 8)] = "Mariä Empfängnis"

        self[date(year, DEC, 25)] = "Weihnachten"

        if self.subdiv in {
            "AG",
            "AI",
            "AR",
            "BE",
            "BL",
            "BS",
            "FR",
            "GL",
            "GR",
            "LU",
            "NE",
            "NW",
            "OW",
            "SG",
            "SH",
            "SO",
            "SZ",
            "TG",
            "TI",
            "UR",
            "ZG",
            "ZH",
        }:
            self[date(year, DEC, 26)] = "Stephanstag"

        if self.subdiv == "GE":
            self[date(year, DEC, 31)] = "Wiederherstellung der Republik"


class CH(Switzerland):
    pass


class CHE(Switzerland):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import MO, TU, WE
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, FEB, MAR, APR, JUN, JUL, SEP, OCT, NOV
from holidays.constants import DEC, FRI
from holidays.holiday_base import HolidayBase


class NewZealand(HolidayBase):

    country = "NZ"
    special_holidays = {2022: ((SEP, 26, "Queen Elizabeth II Memorial Day"),)}
    subdivisions = [
        # https://en.wikipedia.org/wiki/ISO_3166-2:NZ
        "AUK",  # Auckland / Tāmaki-makau-rau
        # "BOP",  # Bay of Plenty / Te Moana a Toi Te Huatahi
        "CAN",  # Canterbury / Waitaha
        # "GIS",  # Gisborne / Tūranga nui a Kiwa
        "HKB",  # Hawke's Bay / Te Matau a Māui
        "MBH",  # Marlborough
        # "MWT",  # Manawatu-Wanganui
        "NSN",  # Nelson / Whakatū
        "NTL",  # Northland / Te Tai tokerau
        "OTA",  # Otago / Ō Tākou
        "STL",  # Southland / Murihiku
        # "TAS",  # Tasman
        "TKI",  # Taranaki
        # "WKO",  # Waikato
        "WGN",  # Wellington / Te Whanga-nui-a-Tara
        "WTC",  # West Coast / Te Taihau ā uru
        "CIT",  # Chatham Islands Territory / Wharekauri
        "South Canterbury",
    ]
    _deprecated_subdivisions = [
        "Auckland",
        "Canterbury",
        "Chatham Islands",
        "Hawke's Bay",
        "Marlborough",
        "Northland",
        "Nelson",
        "Otago",
        "South Canterbury",
        "Southland",
        "Taranaki",
        "Waitangi",
        "Wellington",
        "West Coast",
        "Westland",  # Correct name is West Coast
        "WTL",  # Correct code is WTC
    ]

    @staticmethod
    def _get_nearest_monday(d: date) -> date:
        if d.weekday() < FRI:
            return d + rd(weekday=MO(-1))
        else:
            return d + rd(weekday=MO)

    def _add_observed(self, dt: date) -> None:
        if self.observed and self._is_weekend(dt):
            obs_date = dt + rd(weekday=MO)
            if self.get(obs_date):
                obs_date += rd(days=+1)
            self[obs_date] = f"{self[dt]} (Observed)"

    def _populate(self, year):
        # Bank Holidays Act 1873
        # The Employment of Females Act 1873
        # Factories Act 1894
        # Industrial Conciliation and Arbitration Act 1894
        # Labour Day Act 1899
        # Anzac Day Act 1920, 1949, 1956
        # New Zealand Day Act 1973
        # Waitangi Day Act 1960, 1976
        # Sovereign's Birthday Observance Act 1937, 1952
        # Holidays Act 1981, 2003

        if year <= 1893:
            return
        super()._populate(year)

        # New Year's Day
        jan1 = date(year, JAN, 1)
        self[jan1] = "New Year's Day"

        jan2 = date(year, JAN, 2)
        self[jan2] = "Day after New Year's Day"
        self._add_observed(jan1)
        self._add_observed(jan2)

        # Waitangi Day
        if year >= 1974:
            name = "New Zealand Day"
            if year >= 1977:
                name = "Waitangi Day"
            feb6 = date(year, FEB, 6)
            self[feb6] = name
            if year >= 2014:
                self._add_observed(feb6)

        # Anzac Day
        if year >= 1921:
            apr25 = date(year, APR, 25)
            self[apr25] = "Anzac Day"
            if year >= 2014:
                self._add_observed(apr25)

        # Easter
        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Good Friday"
        self[easter_date + rd(days=+1)] = "Easter Monday"

        # Sovereign's Birthday
        if year >= 1902:
            name = "King's Birthday"
            if 1952 <= year <= 2022:
                name = "Queen's Birthday"
            if year == 1952:
                self[date(year, JUN, 2)] = name  # Elizabeth II
            elif year >= 1938:
                self[date(year, JUN, 1) + rd(weekday=MO)] = name  # EII & GVI
            elif year == 1937:
                self[date(year, JUN, 9)] = name  # George VI
            elif year == 1936:
                self[date(year, JUN, 23)] = name  # Edward VIII
            elif year >= 1912:
                self[date(year, JUN, 3)] = name  # George V
            else:
                # http://paperspast.natlib.govt.nz/cgi-bin/paperspast?a=d&d=NZH19091110.2.67
                self[date(year, NOV, 9)] = name  # Edward VII

        # Matariki
        dates_obs = {
            2022: (JUN, 24),
            2023: (JUL, 14),
            2024: (JUN, 28),
            2025: (JUN, 20),
            2026: (JUL, 10),
            2027: (JUN, 25),
            2028: (JUL, 14),
            2029: (JUL, 6),
            2030: (JUN, 21),
            2031: (JUL, 11),
            2032: (JUL, 2),
            2033: (JUN, 24),
            2034: (JUL, 7),
            2035: (JUN, 29),
            2036: (JUL, 18),
            2037: (JUL, 10),
            2038: (JUN, 25),
            2039: (JUL, 15),
            2040: (JUL, 6),
            2041: (JUL, 19),
            2042: (JUL, 11),
            2043: (JUL, 3),
            2044: (JUN, 24),
            2045: (JUL, 7),
            2046: (JUN, 29),
            2047: (JUL, 19),
            2048: (JUL, 3),
            2049: (JUN, 25),
            2050: (JUL, 15),
            2051: (JUN, 30),
            2052: (JUN, 21),
        }
        if year in dates_obs:
            self[date(year, *dates_obs[year])] = "Matariki"

        # Labour Day
        name = "Labour Day"
        if year >= 1910:
            self[date(year, OCT, 1) + rd(weekday=MO(+4))] = name
        elif year >= 1900:
            self[date(year, OCT, 1) + rd(weekday=WE(+2))] = name

        # Christmas Day
        dec25 = date(year, DEC, 25)
        self[dec25] = "Christmas Day"

        # Boxing Day
        dec26 = date(year, DEC, 26)
        self[dec26] = "Boxing Day"
        self._add_observed(dec25)
        self._add_observed(dec26)

        # Province Anniversary Day
        if self.subdiv in {"Auckland", "AUK", "Northland", "NTL"}:
            if 1964 <= year <= 1973 and self.subdiv in {"Northland", "NTL"}:
                name = "Waitangi Day"
                dt = date(year, FEB, 6)
            else:
                name = "Auckland Anniversary Day"
                dt = date(year, JAN, 29)
            self[self._get_nearest_monday(dt)] = name

        elif self.subdiv in {"New Plymouth", "Taranaki", "TKI"}:
            self[
                date(year, MAR, 1) + rd(weekday=MO(+2))
            ] = "Taranaki Anniversary Day"

        elif self.subdiv in {"Hawke's Bay", "HKB"}:
            self[
                (date(year, OCT, 1) + rd(weekday=MO(+4)) + rd(days=-3))
            ] = "Hawke's Bay Anniversary Day"

        elif self.subdiv in {"WGN", "Wellington"}:
            self[
                self._get_nearest_monday(date(year, JAN, 22))
            ] = "Wellington Anniversary Day"

        elif self.subdiv in {"Marlborough", "MBH"}:
            self[
                (date(year, OCT, 1) + rd(weekday=MO(+4)) + rd(days=+7))
            ] = "Marlborough Anniversary Day"

        elif self.subdiv in {"Nelson", "NSN"}:
            self[
                self._get_nearest_monday(date(year, FEB, 1))
            ] = "Nelson Anniversary Day"

        elif self.subdiv in {"CAN", "Canterbury"}:
            self[
                (date(year, NOV, 1) + rd(weekday=TU) + rd(days=+10))
            ] = "Canterbury Anniversary Day"

        elif self.subdiv in {"South Canterbury", "STC"}:
            self[
                (date(year, SEP, 1) + rd(weekday=MO(+4)))
            ] = "South Canterbury Anniversary Day"

        elif self.subdiv in {"WTC", "West Coast", "WTL", "Westland"}:
            name = "West Coast Anniversary Day"
            # Observance varies?!?!
            if year == 2005:  # special case?!?!
                self[date(year, DEC, 5)] = name
            else:
                self[self._get_nearest_monday(date(year, DEC, 1))] = name

        elif self.subdiv in {"OTA", "Otago"}:
            # there is no easily determined single day of local observance?!?!
            dt = self._get_nearest_monday(date(year, MAR, 23))
            if dt == easter_date + rd(days=+1):  # Avoid Easter Monday
                dt += rd(days=+1)
            self[dt] = "Otago Anniversary Day"

        elif self.subdiv in {"STL", "Southland"}:
            name = "Southland Anniversary Day"
            if year >= 2012:
                self[easter_date + rd(days=+2)] = name
            else:
                self[self._get_nearest_monday(date(year, JAN, 17))] = name

        elif self.subdiv in {"CIT", "Chatham Islands"}:
            self[
                self._get_nearest_monday(date(year, NOV, 30))
            ] = "Chatham Islands Anniversary Day"


class NZ(NewZealand):
    pass


class NZL(NewZealand):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, APR, MAY, AUG, DEC, SUN
from holidays.holiday_base import HolidayBase


class Netherlands(HolidayBase):
    """
    http://www.iamsterdam.com/en/plan-your-trip/practical-info/public-holidays
    """

    country = "NL"

    def _populate(self, year):
        super()._populate(year)

        # New years
        self[date(year, JAN, 1)] = "Nieuwjaarsdag"

        easter_date = easter(year)

        # Easter
        self[easter_date] = "Eerste paasdag"

        # Good friday
        self[easter_date + rd(days=-2)] = "Goede Vrijdag"

        # Second easter day
        self[easter_date + rd(days=+1)] = "Tweede paasdag"

        # Ascension day
        self[easter_date + rd(days=+39)] = "Hemelvaart"

        # Pentecost
        self[easter_date + rd(days=+49)] = "Eerste Pinksterdag"

        # Pentecost monday
        self[easter_date + rd(days=+50)] = "Tweede Pinksterdag"

        # First christmas
        self[date(year, DEC, 25)] = "Eerste Kerstdag"

        # Second christmas
        self[date(year, DEC, 26)] = "Tweede Kerstdag"

        # Liberation day
        if year >= 1945 and year % 5 == 0:
            self[date(year, MAY, 5)] = "Bevrijdingsdag"

        # Kingsday
        if year >= 2014:
            kings_day = date(year, APR, 27)
            if kings_day.weekday() == SUN:
                kings_day = kings_day - rd(days=1)

            self[kings_day] = "Koningsdag"

        # Queen's day
        if 1891 <= year <= 2013:
            queens_day = date(year, APR, 30)
            if year <= 1948:
                queens_day = date(year, AUG, 31)

            if queens_day.weekday() == SUN:
                if year < 1980:
                    queens_day = queens_day + rd(days=1)
                else:
                    queens_day = queens_day - rd(days=1)

            self[queens_day] = "Koninginnedag"


class NL(Netherlands):
    pass


class NLD(Netherlands):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, FEB, MAR, MAY, JUN, AUG, SEP, DEC
from holidays.holiday_base import HolidayBase


class Malta(HolidayBase):
    """
    https://www.gov.mt/en/About%20Malta/Pages/Public%20Holidays.aspx
    """

    country = "MT"

    def _populate(self, year):
        super()._populate(year)

        self[date(year, JAN, 1)] = "New Year"
        self[date(year, FEB, 10)] = "Feast of St. Paul's Shipwreck"
        self[date(year, MAR, 19)] = "Feast of St. Joseph"
        self[date(year, MAR, 31)] = "Freedom Day"

        # Easter and easter related calculations
        good_friday = easter(year) + rd(days=-2)
        self[good_friday] = "Good Friday"

        self[date(year, MAY, 1)] = "Worker's Day"
        self[date(year, JUN, 7)] = "Sette Giugno"
        self[date(year, JUN, 29)] = "Feast of St. Peter and St. Paul"
        self[date(year, AUG, 15)] = "Feast of the Assumption"
        self[date(year, SEP, 8)] = "Feast of Our Lady of Victories"
        self[date(year, SEP, 21)] = "Independence Day"
        self[date(year, DEC, 8)] = "Feast of the Immaculate Conception"
        self[date(year, DEC, 13)] = "Republic Day"
        self[date(year, DEC, 25)] = "Christmas Day"


class MT(Malta):
    pass


class MLT(Malta):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, FEB, APR, OCT
from holidays.holiday_base import HolidayBase
from holidays.utils import _ChineseLuniSolar


class Taiwan(HolidayBase):
    """
    https://en.wikipedia.org/wiki/Public_holidays_in_Taiwan
    """

    country = "TW"

    def __init__(self, **kwargs):
        self.cnls = _ChineseLuniSolar()
        HolidayBase.__init__(self, **kwargs)

    def _populate(self, year):
        super()._populate(year)

        # New Year's Day
        if year > 1911:
            self[
                date(year, JAN, 1)
            ] = "Founding of the Republic of China (New Year's Day)"
            hol_date = self.cnls.lunar_n_y_date(year)
            self[hol_date + rd(days=-1)] = "Chinese New Year's Eve"
            self[hol_date] = "Spring Festival"
            self[hol_date + rd(days=1)] = "Spring Festival"
            self[hol_date + rd(days=2)] = "Spring Festival"
            self[date(year, APR, 4)] = "Children's Day"
            self[self.cnls.lunar_to_gre(year, 5, 5)] = "Dragon Boat Festival"
            self[self.cnls.lunar_to_gre(year, 8, 15)] = "Mid-Autumn Festival"
            self[date(year, OCT, 10)] = "National Day"
            self[date(year, OCT, 11)] = "National Day"
        if year > 1947:
            self[date(year, FEB, 28)] = "Peace Memorial Day"
        if year == 2021:
            hol_date = self.cnls.lunar_n_y_date(year)
            self[hol_date + rd(days=3)] = "Spring Festival"
            self[hol_date + rd(days=4)] = "Spring Festival"


class TW(Taiwan):
    pass


class TWN(Taiwan):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Provinces completed by Henrik Sozzi <henrik_sozzi@hotmail.com>
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd
from dateutil.relativedelta import TU, TH, SU

from holidays.constants import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP
from holidays.constants import OCT, NOV, DEC
from holidays.holiday_base import HolidayBase


class Italy(HolidayBase):

    country = "IT"
    # Reference: https://it.wikipedia.org/wiki/Province_d%27Italia
    # Please maintain in alphabetical order for easy updating in the future
    # The alphabetical order is except cities of provinces with multiple head
    # cities that directly follows the main province id like BT, Barletta,
    # Andria, Trani, for easily grouping them.
    # In that case if you use the 2 char id you'll take the first Santo
    # Patrono defined. If you want one specific you'll have to use
    # the full name of the city like "Andria" instead of "BT".
    subdivisions = [
        "AG",
        "AL",
        "AN",
        "AO",
        "AP",
        "AQ",
        "AR",
        "AT",
        "AV",
        "BA",
        "BG",
        "BI",
        "BL",
        "BN",
        "BO",
        "BR",
        "BS",
        "BT",
        "Barletta",
        "Andria",
        "Trani",
        "BZ",
        "CA",
        "CB",
        "CE",
        "CH",
        "CL",
        "CN",
        "CO",
        "CR",
        "CS",
        "CT",
        "CZ",
        "EN",
        "FC",
        "Forlì",
        "Cesena",
        "FE",
        "FG",
        "FI",
        "FM",
        "FR",
        "GE",
        "GO",
        "GR",
        "IM",
        "IS",
        "KR",
        "LC",
        "LE",
        "LI",
        "LO",
        "LT",
        "LU",
        "MB",
        "MC",
        "ME",
        "MI",
        "MN",
        "MO",
        "MS",
        "MT",
        "NA",
        "NO",
        "NU",
        "OR",
        "PA",
        "PC",
        "PD",
        "PE",
        "PG",
        "PI",
        "PN",
        "PO",
        "PR",
        "PT",
        "PU",
        "Pesaro",
        "Urbino",
        "PV",
        "PZ",
        "RA",
        "RC",
        "RE",
        "RG",
        "RI",
        "RM",
        "RN",
        "RO",
        "SA",
        "SI",
        "SO",
        "SP",
        "SR",
        "SS",
        "SU",
        "SV",
        "TA",
        "TE",
        "TN",
        "TO",
        "TP",
        "TR",
        "TS",
        "TV",
        "UD",
        "VA",
        "VB",
        "VC",
        "VE",
        "VI",
        "VR",
        "VT",
        "VV",
    ]

    def _populate(self, year):
        super()._populate(year)

        self[date(year, JAN, 1)] = "Capodanno"
        self[date(year, JAN, 6)] = "Epifania del Signore"
        easter_date = easter(year)
        self[easter_date] = "Pasqua di Resurrezione"
        self[easter_date + rd(days=+1)] = "Lunedì dell'Angelo"
        if year >= 1946:
            self[date(year, APR, 25)] = "Festa della Liberazione"
        self[date(year, MAY, 1)] = "Festa dei Lavoratori"
        if year >= 1948:
            self[date(year, JUN, 2)] = "Festa della Repubblica"
        self[date(year, AUG, 15)] = "Assunzione della Vergine"
        self[date(year, NOV, 1)] = "Tutti i Santi"
        self[date(year, DEC, 8)] = "Immacolata Concezione"
        self[date(year, DEC, 25)] = "Natale"
        self[date(year, DEC, 26)] = "Santo Stefano"

        # Provinces holidays
        # Reference from:
        # https://it.wikipedia.org/wiki/Santi_patroni_cattolici_delle_citt%C3%A0_capoluogo_di_provincia_italiane
        # Please maintain in alphabetical order for easy updating in the future
        if self.subdiv:
            if self.subdiv == "AG":
                self[date(year, FEB, 25)] = "San Gerlando"
            elif self.subdiv == "AL":
                self[date(year, NOV, 10)] = "San Baudolino"
            elif self.subdiv == "AN":
                self[date(year, MAY, 4)] = "San Ciriaco"
            elif self.subdiv == "AO":
                self[date(year, SEP, 7)] = "San Grato"
            elif self.subdiv == "AP":
                self[date(year, AUG, 5)] = "Sant'Emidio"
            elif self.subdiv == "AQ":
                self[date(year, JUN, 10)] = "San Massimo D'Aveia"
            elif self.subdiv == "AR":
                self[date(year, AUG, 7)] = "San Donato D'Arezzo"
            elif self.subdiv == "AT":
                self[
                    date(year, MAY, 1) + rd(weekday=TU)
                ] = "San Secondo di Asti"  # <--- First Tuesday in May
            elif self.subdiv == "AV":
                self[date(year, FEB, 14)] = "San Modestino"
            elif self.subdiv == "BA":
                self[date(year, DEC, 6)] = "San Nicola"
            elif self.subdiv == "BG":
                self[date(year, AUG, 26)] = "Sant'Alessandro di Bergamo"
            elif self.subdiv == "BI":
                self[date(year, DEC, 26)] = "Santo Stefano"
            elif self.subdiv == "BL":
                self[date(year, NOV, 11)] = "San Martino"
            elif self.subdiv == "BN":
                self[date(year, AUG, 24)] = "San Bartolomeo apostolo"
            elif self.subdiv == "BO":
                self[date(year, OCT, 4)] = "San Petronio"
            elif self.subdiv == "BR":
                # first Sunday of September
                self[
                    date(year, SEP, 1) + rd(weekday=SU)
                ] = "San Teodoro d'Amasea e San Lorenzo da Brindisi"
            elif self.subdiv == "BS":
                self[date(year, FEB, 15)] = "Santi Faustino e Giovita"
            elif self.subdiv in {"BT", "Barletta"}:
                self[date(year, DEC, 30)] = "San Ruggero"
            if self.subdiv in {"BT", "Andria"}:
                self[
                    date(year, SEP, 1) + rd(weekday=SU(+3))
                ] = "San Riccardo di Andria"  # <--- Third sunday in September
            if self.subdiv in {"BT", "Trani"}:
                self[date(year, MAY, 3)] = "San Nicola Pellegrino"
            elif self.subdiv == "BZ":
                self[easter_date + rd(days=+50)] = "Lunedì di Pentecoste"
                self[date(year, AUG, 15)] = "Maria Santissima Assunta"
            elif self.subdiv == "CA":
                self[date(year, OCT, 30)] = "San Saturnino di Cagliari"
            elif self.subdiv == "CB":
                self[date(year, APR, 23)] = "San Giorgio"
            elif self.subdiv == "CE":
                self[date(year, JAN, 20)] = "San Sebastiano"
            elif self.subdiv == "CH":
                self[date(year, MAY, 11)] = "San Giustino di Chieti"
            elif self.subdiv == "CL":
                self[date(year, SEP, 29)] = "San Michele Arcangelo"
            elif self.subdiv == "CN":
                self[date(year, SEP, 29)] = "San Michele Arcangelo"
            elif self.subdiv == "CO":
                self[date(year, AUG, 31)] = "Sant'Abbondio"
            elif self.subdiv == "CR":
                self[date(year, NOV, 13)] = "Sant'Omobono"
            elif self.subdiv == "CS":
                self[date(year, FEB, 12)] = "Madonna del Pilerio"
            elif self.subdiv == "CT":
                self[date(year, FEB, 5)] = "Sant'Agata"
            elif self.subdiv == "CZ":
                self[date(year, JUL, 16)] = "San Vitaliano"
            elif self.subdiv == "EN":
                self[date(year, JUL, 2)] = "Madonna della Visitazione"
            elif self.subdiv in {"FC", "Cesena"}:
                self[date(year, JUN, 24)] = "San Giovanni Battista"
            if self.subdiv in {"FC", "Forlì"}:
                self[date(year, FEB, 4)] = "Madonna del Fuoco"
            elif self.subdiv == "FE":
                self[date(year, APR, 23)] = "San Giorgio"
            elif self.subdiv == "FG":
                self[date(year, MAR, 22)] = "Madonna dei Sette Veli"
            elif self.subdiv == "FI":
                self[date(year, JUN, 24)] = "San Giovanni Battista"
            elif self.subdiv == "FM":
                self[date(year, AUG, 15)] = "Maria Santissima Assunta"
                self[date(year, AUG, 16)] = "Maria Santissima Assunta"
            elif self.subdiv == "FR":
                self[date(year, JUN, 20)] = "San Silverio"
            elif self.subdiv == "GE":
                self[date(year, JUN, 24)] = "San Giovanni Battista"
            elif self.subdiv == "GO":
                self[date(year, MAR, 16)] = "Santi Ilario e Taziano"
            elif self.subdiv == "GR":
                self[date(year, AUG, 10)] = "San Lorenzo"
            elif self.subdiv == "IM":
                self[date(year, NOV, 26)] = "San Leonardo da Porto Maurizio"
            elif self.subdiv == "IS":
                self[date(year, MAY, 19)] = "San Pietro Celestino"
            elif self.subdiv == "KR":
                self[date(year, OCT, 9)] = "San Dionigi"
            elif self.subdiv == "LC":
                self[date(year, DEC, 6)] = "San Nicola"
            elif self.subdiv == "LE":
                self[date(year, AUG, 26)] = "Sant'Oronzo"
            elif self.subdiv == "LI":
                self[date(year, MAY, 22)] = "Santa Giulia"
            elif self.subdiv == "LO":
                self[date(year, JAN, 19)] = "San Bassiano"
            elif self.subdiv == "LT":
                self[date(year, APR, 25)] = "San Marco evangelista"
            elif self.subdiv == "LU":
                self[date(year, JUL, 12)] = "San Paolino di Lucca"
            elif self.subdiv == "MB":
                self[date(year, JUN, 24)] = "San Giovanni Battista"
            elif self.subdiv == "MC":
                self[date(year, AUG, 31)] = "San Giuliano l'ospitaliere"
            elif self.subdiv == "ME":
                self[date(year, JUN, 3)] = "Madonna della Lettera"
            elif self.subdiv == "MI":
                self[date(year, DEC, 7)] = "Sant'Ambrogio"
            elif self.subdiv == "MN":
                self[date(year, MAR, 18)] = "Sant'Anselmo da Baggio"
            elif self.subdiv == "MO":
                self[date(year, JAN, 31)] = "San Geminiano"
            elif self.subdiv == "MS":
                self[date(year, OCT, 4)] = "San Francesco d'Assisi"
            elif self.subdiv == "MT":
                self[date(year, JUL, 2)] = "Madonna della Bruna"
            elif self.subdiv == "NA":
                self[date(year, SEP, 19)] = "San Gennaro"
            elif self.subdiv == "NO":
                self[date(year, JAN, 22)] = "San Gaudenzio"
            elif self.subdiv == "NU":
                self[date(year, AUG, 5)] = "Nostra Signora della Neve"
            elif self.subdiv == "OR":
                self[date(year, FEB, 13)] = "Sant'Archelao"
            elif self.subdiv == "PA":
                self[date(year, JUL, 15)] = "San Giovanni"
            elif self.subdiv == "PC":
                self[date(year, JUL, 4)] = "Sant'Antonino di Piacenza"
            elif self.subdiv == "PD":
                self[date(year, JUN, 13)] = "Sant'Antonio di Padova"
            elif self.subdiv == "PE":
                self[date(year, OCT, 10)] = "San Cetteo"
            elif self.subdiv == "PG":
                self[date(year, JAN, 29)] = "Sant'Ercolano e San Lorenzo"
            elif self.subdiv == "PI":
                self[date(year, JUN, 17)] = "San Ranieri"
            elif self.subdiv == "PN":
                self[date(year, APR, 25)] = "San Marco Evangelista"
                self[date(year, SEP, 8)] = "Madonna delle Grazie"
            elif self.subdiv == "PO":
                self[date(year, DEC, 26)] = "Santo Stefano"
            elif self.subdiv == "PR":
                self[date(year, JAN, 13)] = "Sant'Ilario di Poitiers"
            elif self.subdiv == "PT":
                self[date(year, JUL, 25)] = "San Jacopo"
            elif self.subdiv in {"PU", "Pesaro"}:
                self[date(year, SEP, 24)] = "San Terenzio di Pesaro"
            if self.subdiv in {"PU", "Urbino"}:
                self[date(year, JUN, 1)] = "San Crescentino"
            elif self.subdiv == "PV":
                self[date(year, DEC, 9)] = "San Siro"
            elif self.subdiv == "PZ":
                self[date(year, MAY, 30)] = "San Gerardo di Potenza"
            elif self.subdiv == "RA":
                self[date(year, JUL, 23)] = "Sant'Apollinare"
            elif self.subdiv == "RC":
                self[date(year, APR, 23)] = "San Giorgio"
            elif self.subdiv == "RE":
                self[date(year, NOV, 24)] = "San Prospero Vescovo"
            elif self.subdiv == "RG":
                self[date(year, APR, 23)] = "San Giorgio"
            elif self.subdiv == "RI":
                self[date(year, DEC, 4)] = "Santa Barbara"
            elif self.subdiv == "RM":
                self[date(year, JUN, 29)] = "Santi Pietro e Paolo"
            elif self.subdiv == "RN":
                self[date(year, OCT, 14)] = "San Gaudenzio"
            elif self.subdiv == "RO":
                self[date(year, NOV, 26)] = "San Bellino"
            elif self.subdiv == "SA":
                self[date(year, SEP, 21)] = "San Matteo Evangelista"
            elif self.subdiv == "SI":
                self[date(year, DEC, 1)] = "Sant'Ansano"
            elif self.subdiv == "SO":
                self[date(year, JUN, 19)] = "San Gervasio e San Protasio"
            elif self.subdiv == "SP":
                self[date(year, MAR, 19)] = "San Giuseppe"
            elif self.subdiv == "SR":
                self[date(year, DEC, 13)] = "Santa Lucia"
            elif self.subdiv == "SS":
                self[date(year, DEC, 6)] = "San Nicola"
            elif self.subdiv == "SU":
                self[
                    date(year, MAY, 1) + rd(weekday=SU(+2)) + rd(weekday=TH)
                ] = "San Ponziano"  # <--- Thursday after second sunday in May
            elif self.subdiv == "SV":
                self[date(year, MAR, 18)] = "Nostra Signora della Misericordia"
            elif self.subdiv == "TA":
                self[date(year, MAY, 10)] = "San Cataldo"
            elif self.subdiv == "TE":
                self[date(year, DEC, 19)] = "San Berardo da Pagliara"
            elif self.subdiv == "TN":
                self[date(year, JUN, 26)] = "San Vigilio"
            elif self.subdiv == "TO":
                self[date(year, JUN, 24)] = "San Giovanni Battista"
            elif self.subdiv == "TP":
                self[date(year, AUG, 7)] = "Sant'Alberto degli Abati"
            elif self.subdiv == "TR":
                self[date(year, FEB, 14)] = "San Valentino"
            elif self.subdiv == "TS":
                self[date(year, NOV, 3)] = "San Giusto"
            elif self.subdiv == "TV":
                self[date(year, APR, 27)] = "San Liberale"
            elif self.subdiv == "UD":
                self[date(year, JUL, 12)] = "Santi Ermacora e Fortunato"
            elif self.subdiv == "VA":
                self[date(year, MAY, 8)] = "San Vittore il Moro"
            elif self.subdiv == "VB":
                self[date(year, MAY, 8)] = "San Vittore il Moro"
            elif self.subdiv == "VC":
                self[date(year, AUG, 1)] = "Sant'Eusebio di Vercelli"
            elif self.subdiv == "VE":
                self[date(year, APR, 25)] = "San Marco Evangelista"
            elif self.subdiv == "VI":
                self[date(year, APR, 25)] = "San Marco"
            elif self.subdiv == "VR":
                self[date(year, MAY, 21)] = "San Zeno"
            elif self.subdiv == "VT":
                self[date(year, SEP, 4)] = "Santa Rosa da Viterbo"
            elif self.subdiv == "VV":
                self[date(year, MAR, 1)] = "San Leoluca"


class IT(Italy):
    pass


class ITA(Italy):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)
from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import MO
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, FEB, MAR, MAY, JUN, AUG, OCT, DEC, FRI
from holidays.holiday_base import HolidayBase


class Ireland(HolidayBase):
    """
    Official holidays in Ireland, as declared in the Citizen's Information
    bulletin:
    https://www.citizensinformation.ie/en/employment/employment_rights_and_conditions/leave_and_holidays/public_holidays_in_ireland.html
    """

    country = "IE"
    special_holidays = {
        2022: ((MAR, 18, "Day of Remembrance and Recognition"),)
    }

    def _populate(self, year):
        super()._populate(year)

        self[date(year, JAN, 1)] = "New Year's Day"

        # St. Brigid's Day
        if year >= 2023:
            name = "St. Brigid's Day"
            dt = date(year, FEB, 1)
            self[dt] = name
            if self.observed and dt.weekday() != FRI:
                self[dt + rd(weekday=MO)] = name + " (Observed)"

        # St. Patrick's Day
        name = "St. Patrick's Day"
        dt = date(year, MAR, 17)
        self[dt] = name
        if self.observed and self._is_weekend(dt):
            self[dt + rd(weekday=MO)] = name + " (Observed)"

        # Easter Monday
        self[easter(year) + rd(days=+1)] = "Easter Monday"

        # May bank holiday (first Monday in May)
        if year >= 1978:
            name = "May Day"
            if year == 1995:
                dt = date(year, MAY, 8)
            else:
                dt = date(year, MAY, 1) + rd(weekday=MO)
            self[dt] = name

        # June bank holiday (first Monday in June)
        self[date(year, JUN, 1) + rd(weekday=MO)] = "June Bank Holiday"

        # Summer bank holiday (first Monday in August)
        self[date(year, AUG, 1) + rd(weekday=MO)] = "August Bank Holiday"

        # October Bank Holiday (last Monday in October)
        self[date(year, OCT, 31) + rd(weekday=MO(-1))] = "October Bank Holiday"

        # Christmas Day
        name = "Christmas Day"
        dt = date(year, DEC, 25)
        self[dt] = name
        if self.observed and self._is_weekend(dt):
            self[dt + rd(weekday=MO)] = name + " (Observed)"

        # St. Stephen's Day
        name = "St. Stephen's Day"
        dt = date(year, DEC, 26)
        self[dt] = name
        if self.observed and self._is_weekend(dt):
            self[dt + rd(days=2)] = name + " (Observed)"


class IE(Ireland):
    pass


class IRL(Ireland):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import MO
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAY, JUL, SEP, OCT, DEC, SAT, SUN
from holidays.holiday_base import HolidayBase


class Botswana(HolidayBase):
    """
    https://www.gov.bw/public-holidays
    https://publicholidays.africa/botswana/2021-dates/
    https://www.timeanddate.com/holidays/botswana/
    http://www.ilo.org/dyn/travail/docs/1766/Public%20Holidays%20Act.pdf
    """

    country = "BW"
    special_holidays = {2019: ((JUL, 2, "Public Holiday"),)}

    def _populate(self, year: int):
        if year <= 1965:
            return
        super()._populate(year)

        self[date(year, JAN, 1)] = "New Year's Day"
        self[date(year, JAN, 2)] = "New Year's Day Holiday"

        # Easter and easter related calculations
        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Good Friday"
        self[easter_date + rd(days=-1)] = "Holy Saturday"
        self[easter_date + rd(days=+1)] = "Easter Monday"

        self[date(year, MAY, 1)] = "Labour Day"
        self[easter_date + rd(days=+39)] = "Ascension Day"

        self[date(year, JUL, 1)] = "Sir Seretse Khama Day"

        # 3rd Monday of July = "President's Day"
        d = date(year, JUL, 1) + rd(weekday=MO(+3))
        self[d] = "President's Day"
        self[d + rd(days=1)] = "President's Day Holiday"

        self[date(year, SEP, 30)] = "Botswana Day"
        self[date(year, OCT, 1)] = "Botswana Day Holiday"

        self[date(year, DEC, 25)] = "Christmas Day"
        self[date(year, DEC, 26)] = "Boxing Day"

        if self.observed:
            for k, v in list(self.items()):
                # Whenever Boxing Day falls on a Saturday,
                # it rolls over to the following Monday
                if (
                    2016 <= year == k.year
                    and k.weekday() == SAT
                    and v.upper() in {"BOXING DAY", "LABOUR DAY"}
                ):
                    # Add the (Observed) holiday
                    self[k + rd(days=+2)] = v + " Holiday"
                if (
                    1995 <= year == k.year
                    and k.weekday() == SUN
                    and v.upper() != "NEW YEAR'S DAY HOLIDAY"
                ):
                    # Add the (Observed) holiday
                    self[k + rd(days=+1)] = v + " (Observed)"

                # If there is a holiday and an (Observed) holiday
                # on the same day, add an (Observed) holiday for that holiday
                hol_names = self.get(k).split(",")
                if len(hol_names) > 1:
                    # self.get(date) returns a string containing holidays as a
                    # comma delimited string split on delimiter to determine if
                    # there are multiple on the same day
                    # Add an (Observed) for the one that is not (Observed)
                    for name in hol_names:
                        if " (Observed)" not in name:
                            self[k + rd(days=+1)] = (
                                name.lstrip() + " (Observed)"
                            )


class BW(Botswana):
    pass


class BWA(Botswana):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.holiday_base import HolidayBase


class France(HolidayBase):
    """Official French holidays.

    Some provinces have specific holidays, only those are included in the
    PROVINCES, because these provinces have different administrative status,
    which makes it difficult to enumerate.

    For religious holidays usually happening on Sundays (Easter, Pentecost),
    only the following Monday is considered a holiday.

    Primary sources:
        https://fr.wikipedia.org/wiki/Fêtes_et_jours_fériés_en_France
        https://www.service-public.fr/particuliers/vosdroits/F2405
    """

    country = "FR"
    subdivisions = [
        "Métropole",
        "Alsace-Moselle",
        "Guadeloupe",
        "Guyane",
        "Martinique",
        "Mayotte",
        "Nouvelle-Calédonie",
        "La Réunion",
        "Polynésie Française",
        "Saint-Barthélémy",
        "Saint-Martin",
        "Wallis-et-Futuna",
    ]

    def __init__(self, **kwargs):
        # Default subdivision to Métropole; prov for backwards compatibility
        if not kwargs.get("subdiv", kwargs.get("prov")):
            kwargs["subdiv"] = "Métropole"
        HolidayBase.__init__(self, **kwargs)

    def _populate(self, year):
        super()._populate(year)

        # Civil holidays
        if year > 1810:
            self[date(year, JAN, 1)] = "Jour de l'an"

        if year > 1919:
            name = "Fête du Travail"
            if year <= 1948:
                name += " et de la Concorde sociale"
            self[date(year, MAY, 1)] = name

        if (1953 <= year <= 1959) or year > 1981:
            self[date(year, MAY, 8)] = "Armistice 1945"

        if year >= 1880:
            self[date(year, JUL, 14)] = "Fête nationale"

        if year >= 1918:
            self[date(year, NOV, 11)] = "Armistice 1918"

        # Religious holidays
        easter_date = easter(year)

        if self.subdiv in {
            "Alsace-Moselle",
            "Guadeloupe",
            "Guyane",
            "Martinique",
            "Polynésie Française",
        }:
            self[easter_date + rd(days=-2)] = "Vendredi saint"

        if self.subdiv == "Alsace-Moselle":
            self[date(year, DEC, 26)] = "Deuxième jour de Noël"

        if year >= 1886:
            self[easter_date + rd(days=+1)] = "Lundi de Pâques"
            self[easter_date + rd(days=+50)] = "Lundi de Pentecôte"

        if year >= 1802:
            self[easter_date + rd(days=+39)] = "Ascension"
            self[date(year, AUG, 15)] = "Assomption"
            self[date(year, NOV, 1)] = "Toussaint"

            name = "Noël"
            if self.subdiv == "Alsace-Moselle":
                name = "Premier jour de " + name
            self[date(year, DEC, 25)] = name

        # Non-metropolitan holidays (starting dates missing)
        if self.subdiv == "Mayotte":
            self[date(year, APR, 27)] = "Abolition de l'esclavage"

        if self.subdiv == "Wallis-et-Futuna":
            self[date(year, APR, 28)] = "Saint Pierre Chanel"

        if self.subdiv == "Martinique":
            self[date(year, MAY, 22)] = "Abolition de l'esclavage"

        if self.subdiv in {"Guadeloupe", "Saint-Martin"}:
            self[date(year, MAY, 27)] = "Abolition de l'esclavage"

        if self.subdiv == "Guyane":
            self[date(year, JUN, 10)] = "Abolition de l'esclavage"

        if self.subdiv == "Polynésie Française":
            self[date(year, JUN, 29)] = "Fête de l'autonomie"

        if self.subdiv in {"Guadeloupe", "Martinique"}:
            self[date(year, JUL, 21)] = "Fête Victor Schoelcher"

        if self.subdiv == "Wallis-et-Futuna":
            self[date(year, JUL, 29)] = "Fête du Territoire"

        if self.subdiv == "Nouvelle-Calédonie":
            self[date(year, SEP, 24)] = "Fête de la Citoyenneté"

        if self.subdiv == "Saint-Barthélémy":
            self[date(year, OCT, 9)] = "Abolition de l'esclavage"

        if self.subdiv == "La Réunion" and year >= 1981:
            self[date(year, DEC, 20)] = "Abolition de l'esclavage"


# *Warning* FR is also used by dateutlis (Friday), so be careful with its use
class FR(France):
    pass


class FRA(France):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import EASTER_ORTHODOX, easter
from dateutil.relativedelta import MO, TU
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAR, MAY, AUG, OCT, DEC
from holidays.holiday_base import HolidayBase


class Greece(HolidayBase):
    """
    https://en.wikipedia.org/wiki/Public_holidays_in_Greece
    """

    country = "GR"

    def _populate(self, year):
        super()._populate(year)

        easter_date = easter(year, method=EASTER_ORTHODOX)

        # New Years
        self[date(year, JAN, 1)] = "Πρωτοχρονιά [New Year's Day]"
        # Epiphany
        self[date(year, JAN, 6)] = "Θεοφάνεια [Epiphany]"

        # Clean Monday
        self[easter_date + rd(days=-48)] = "Καθαρά Δευτέρα [Clean Monday]"

        # Independence Day
        self[date(year, MAR, 25)] = "Εικοστή Πέμπτη Μαρτίου [Independence Day]"

        # Easter Monday
        self[easter_date + rd(days=+1)] = "Δευτέρα του Πάσχα [Easter Monday]"

        # Monday of the Holy Spirit
        self[
            easter_date + rd(days=+50)
        ] = "Δευτέρα του Αγίου Πνεύματος [Monday of the Holy Spirit]"

        # Labour Day
        name = "Εργατική Πρωτομαγιά [Labour day]"
        name_observed = name + " (Observed)"

        self[date(year, MAY, 1)] = name
        if self.observed and self._is_weekend(year, MAY, 1):
            # https://en.wikipedia.org/wiki/Public_holidays_in_Greece
            labour_day_observed_date = date(year, MAY, 1) + rd(weekday=MO)
            # In 2016 and 2021, Labour Day coincided with other holidays
            # https://www.timeanddate.com/holidays/greece/labor-day
            if self.get(labour_day_observed_date):
                labour_day_observed_date += rd(weekday=TU)
            self[labour_day_observed_date] = name_observed

        # Assumption of Mary
        self[date(year, AUG, 15)] = "Κοίμηση της Θεοτόκου [Assumption of Mary]"

        # Ochi Day
        self[date(year, OCT, 28)] = "Ημέρα του Όχι [Ochi Day]"

        # Christmas
        self[date(year, DEC, 25)] = "Χριστούγεννα [Christmas]"

        # Day after Christmas
        self[
            date(year, DEC, 26)
        ] = "Επόμενη ημέρα των Χριστουγέννων [Day after Christmas]"


class GR(Greece):
    pass


class GRC(Greece):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import EASTER_ORTHODOX, easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAR, APR, MAY, AUG, OCT, DEC
from holidays.holiday_base import HolidayBase


class Cyprus(HolidayBase):
    """
    https://en.wikipedia.org/wiki/Public_holidays_in_Cyprus
    """

    country = "CY"

    def _populate(self, year):
        super()._populate(year)

        easter_date = easter(year, method=EASTER_ORTHODOX)

        # New Years
        self[date(year, JAN, 1)] = "Πρωτοχρονιά [New Year's Day]"

        # Epiphany
        self[date(year, JAN, 6)] = "Θεοφάνεια [Epiphany]"

        # Clean Monday
        self[easter_date + rd(days=-48)] = "Καθαρά Δευτέρα [Clean Monday]"

        # Greek Independence Day
        self[
            date(year, MAR, 25)
        ] = "Εικοστή Πέμπτη Μαρτίου [Greek Independence Day]"

        # Cyprus National Day
        self[date(year, APR, 1)] = "1η Απριλίου [Cyprus National Day]"

        # Good Friday
        self[easter_date + rd(days=-2)] = "Μεγάλη Παρασκευή [Good Friday]"

        # Easter Sunday
        self[easter_date] = "Κυριακή του Πάσχα [Easter Sunday]"

        # Easter Monday
        self[easter_date + rd(days=+1)] = "Δευτέρα του Πάσχα [Easter Monday]"

        # Labour Day
        self[date(year, MAY, 1)] = "Εργατική Πρωτομαγιά [Labour day]"

        # Monday of the Holy Spirit
        self[
            easter_date + rd(days=+50)
        ] = "Δευτέρα του Αγίου Πνεύματος [Monday of the Holy Spirit]"

        # Assumption of Mary
        self[date(year, AUG, 15)] = "Κοίμηση της Θεοτόκου [Assumption of Mary]"

        # Cyprus Independence Day
        self[
            date(year, OCT, 1)
        ] = "Ημέρα Ανεξαρτησίας της Κύπρου [Cyprus Independence Day]"

        # Ochi Day
        self[date(year, OCT, 28)] = "Ημέρα του Όχι [Ochi Day]"

        # Christmas Eve
        self[date(year, DEC, 24)] = "Παραμονή Χριστουγέννων [Christmas Eve]"

        # Christmas
        self[date(year, DEC, 25)] = "Χριστούγεννα [Christmas]"

        # Day after Christmas
        self[
            date(year, DEC, 26)
        ] = "Δεύτερη μέρα Χριστουγέννων [Day after Christmas]"


class CY(Cyprus):
    pass


class CYP(Cyprus):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAR, APR, MAY, JUL, AUG, OCT
from holidays.holiday_base import HolidayBase
from holidays.utils import _islamic_to_gre


class Tunisia(HolidayBase):
    country = "TN"

    # Holidays here are estimates, it is common for the day to be pushed
    # if falls in a weekend, although not a rule that can be implemented.
    # Holidays after 2020: the following four moving date holidays whose exact
    # date is announced yearly are estimated (and so denoted):
    # - Eid El Fetr*
    # - Eid El Adha*
    # - Arafat Day*
    # - Moulad El Naby*
    # *only if hijri-converter library is installed, otherwise a warning is
    #  raised that this holiday is missing. hijri-converter requires
    #  Python >= 3.6
    # is_weekend function is there, however not activated for accuracy.

    def _populate(self, year):
        super()._populate(year)

        def _add_holiday(dt: date, hol: str) -> None:
            """Only add if in current year; prevents adding holidays across
            years (handles multi-day Islamic holidays that straddle Gregorian
            years).
            """
            if dt.year == year:
                self[dt] = hol

        # New Year's Day
        self[date(year, JAN, 1)] = "New Year's Day"

        # Revolution and Youth Day - January 14
        self[date(year, JAN, 14)] = "Revolution and Youth Day - January 14"

        # Independence Day
        self[date(year, MAR, 20)] = "Independence Day"

        # Martyrs' Day
        self[date(year, APR, 9)] = "Martyrs' Day"

        # Labour Day
        self[date(year, MAY, 1)] = "Labour Day"

        # Republic Day
        self[date(year, JUL, 25)] = "Republic Day"

        # Women's Day
        self[date(year, AUG, 13)] = "Women's Day"

        # Evacuation Day
        self[date(year, OCT, 15)] = "Evacuation Day"

        # Eid al-Fitr - Feast Festive
        # date of observance is announced yearly, This is an estimate since
        # having the Holiday on Weekend does change the number of days,
        # deceided to leave it since marking a Weekend as a holiday
        # wouldn't do much harm.
        for yr in (year - 1, year):
            for date_obs in _islamic_to_gre(yr, 10, 1):
                hol_date = date_obs
                _add_holiday(hol_date, "Eid al-Fitr")
                _add_holiday(hol_date + rd(days=1), "Eid al-Fitr Holiday")
                _add_holiday(hol_date + rd(days=2), "Eid al-Fitr Holiday")

        # Arafat Day & Eid al-Adha - Scarfice Festive
        # date of observance is announced yearly
        for yr in (year - 1, year):
            for date_obs in _islamic_to_gre(yr, 12, 9):
                hol_date = date_obs
                _add_holiday(hol_date, "Arafat Day")
                _add_holiday(hol_date + rd(days=1), "Eid al-Adha")
                _add_holiday(hol_date + rd(days=2), "Eid al-Adha Holiday")
                _add_holiday(hol_date + rd(days=3), "Eid al-Adha Holiday")

        # Islamic New Year - (hijari_year, 1, 1)
        for date_obs in _islamic_to_gre(year, 1, 1):
            hol_date = date_obs
            self[hol_date] = "Islamic New Year"

        # Prophet Muhammad's Birthday - (hijari_year, 3, 12)
        for date_obs in _islamic_to_gre(year, 3, 12):
            hol_date = date_obs
            self[hol_date] = "Prophet Muhammad's Birthday"


class TN(Tunisia):
    pass


class TUN(Tunisia):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import EASTER_ORTHODOX, easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, APR, MAR, MAY, JUN, JUL, AUG, SEP, OCT
from holidays.constants import NOV, DEC
from holidays.holiday_base import HolidayBase


class Ukraine(HolidayBase):
    """
    Current holidays list:
    https://zakon1.rada.gov.ua/laws/show/322-08/paran454#n454
    """

    country = "UA"

    def _populate(self, year):
        # The current set of holidays came into force in 1991
        # But most holiday days were implemented in 1918
        if year <= 1917:
            return
        super()._populate(year)

        # New Year's Day
        if year <= 1929 or year >= 1948:
            self[date(year, JAN, 1)] = "Новий рік"

        # Christmas Day (Julian calendar)
        if year >= 1991:
            self[
                date(year, JAN, 7)
            ] = "Різдво Христове (за юліанським календарем)"

        # Women's Day
        if year >= 1966:
            self[date(year, MAR, 8)] = "Міжнародний жіночий день"

        if year >= 1991:
            # Easter
            easter_date = easter(year, method=EASTER_ORTHODOX)
            self[easter_date] = "Великдень (Пасха)"

            # Holy trinity
            self[easter_date + rd(days=+49)] = "Трійця"

        # Labour Day
        name = "День міжнародної солідарності трудящих"
        if year >= 2018:
            name = "День праці"
        self[date(year, MAY, 1)] = name

        # Labour Day in past
        if 1929 <= year <= 2017:
            self[date(year, MAY, 2)] = "День міжнародної солідарності трудящих"

        # Victory Day
        name = "День перемоги"
        dt = date(year, MAY, 9)
        if year >= 2016:
            name = (
                "День перемоги над нацизмом у Другій світовій війні "
                "(День перемоги)"
            )
        if year >= 1965:
            self[dt] = name
        elif 1945 <= year <= 1946:
            self[dt] = name
            self[date(year, SEP, 3)] = "День перемоги над Японією"

        # Constitution Day
        if year >= 1997:
            self[date(year, JUN, 28)] = "День Конституції України"

        # Day of Ukrainian Statehood
        if year >= 2022:
            self[date(year, JUL, 28)] = "День Української Державності"

        # Independence Day
        name = "День незалежності України"
        if year >= 1992:
            self[date(year, AUG, 24)] = name
        elif year == 1991:
            self[date(year, JUL, 16)] = name

        # Day of the defender of Ukraine
        if year >= 2015:
            name = "День захисника України"
            if year >= 2021:
                name = "День захисників і захисниць України"
            self[date(year, OCT, 14)] = name

        # October Revolution
        if year <= 1999:
            name = "Річниця Великої Жовтневої соціалістичної революції"
            self[date(year, NOV, 7)] = name
            self[date(year, NOV, 8)] = name

        # Christmas Day (Gregorian calendar)
        if year >= 2017:
            self[
                date(year, DEC, 25)
            ] = "Різдво Христове (за григоріанським календарем)"

        # 27.01.1995: holiday on weekend move to next workday
        # https://zakon.rada.gov.ua/laws/show/35/95-вр
        # 10.01.1998: cancelled
        # https://zakon.rada.gov.ua/laws/show/785/97-вр
        # 23.04.1999: holiday on weekend move to next workday
        # https://zakon.rada.gov.ua/laws/show/576-14
        if self.observed:
            for k, v in list(self.items()):
                if (
                    self._is_weekend(k)
                    and k.year == year
                    and (
                        date(1995, JAN, 27) <= k <= date(1998, JAN, 9)
                        or k >= date(1999, APR, 23)
                    )
                ):
                    next_workday = k + rd(days=+1)
                    while self._is_weekend(next_workday) or self.get(
                        next_workday
                    ):
                        next_workday += rd(days=+1)
                    self[next_workday] = "Вихідний за " + v

        # USSR holidays
        # Bloody_Sunday_(1905)
        if year <= 1950:
            self[date(year, JAN, 22)] = "День пам’яті 9 січня 1905 року"

        # Paris_Commune
        if year <= 1928:
            self[date(year, MAR, 18)] = "День Паризької Комуни"

        # USSR Constitution day
        name = "День Конституції СРСР"
        if 1981 <= year <= 1990:
            self[date(year, OCT, 7)] = name
        elif 1937 <= year <= 1980:
            self[date(year, DEC, 5)] = name


class UA(Ukraine):
    pass


class UKR(Ukraine):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

import warnings
from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, APR, MAY, JUL, SEP, DEC, SUN
from holidays.holiday_base import HolidayBase


class Eswatini(HolidayBase):
    """
    https://swazilii.org/sz/legislation/act/1938/71
    https://www.officeholidays.com/countries/swaziland
    """

    country = "SZ"
    special_holidays = {
        # https://mg.co.za/article/1999-12-09-swaziland-declares-bank-holidays/
        1999: ((DEC, 31, "Y2K changeover"),),
        2000: ((JAN, 3, "Y2K changeover"),),
    }

    def _populate(self, year):
        # Observed since 1939
        if year <= 1938:
            return
        super()._populate(year)

        self[date(year, JAN, 1)] = "New Year's Day"

        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Good Friday"
        self[easter_date + rd(days=+1)] = "Easter Monday"
        self[easter_date + rd(days=+39)] = "Ascension Day"

        if year >= 1969:
            self[date(year, APR, 25)] = "National Flag Day"

        if year >= 1983:
            # https://www.officeholidays.com/holidays/swaziland/birthday-of-late-king-sobhuza
            self[date(year, JUL, 22)] = "Birthday of Late King Sobhuza"

        if year >= 1987:
            # https://www.officeholidays.com/holidays/swaziland/birthday-of-king-mswati-iii
            self[date(year, APR, 19)] = "King's Birthday"

        self[date(year, MAY, 1)] = "Worker's Day"
        self[date(year, SEP, 6)] = "Independence Day"
        self[date(year, DEC, 25)] = "Christmas Day"
        self[date(year, DEC, 26)] = "Boxing Day"

        # As of 2021/1/1, whenever a public holiday falls on a
        # Sunday
        # it rolls over to the following Monday
        if self.observed and year >= 2021:
            for k, v in list(self.items()):
                if k.weekday() == SUN and k.year == year:
                    dt = k + rd(days=+1)
                    while self.get(dt):
                        dt += rd(days=+1)
                    self[dt] = v + " (Observed)"


class Swaziland(Eswatini):
    def __init__(self, *args, **kwargs) -> None:
        warnings.warn(
            "Swaziland is deprecated, use Eswatini instead.",
            DeprecationWarning,
        )

        super().__init__(*args, **kwargs)


class SZ(Eswatini):
    pass


class SZW(Eswatini):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import WE
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, APR, MAY, SEP, OCT, DEC
from holidays.holiday_base import HolidayBase


class Honduras(HolidayBase):
    # Artículo 339 del Código del Trabajo:
    # https://www.ilo.org/dyn/natlex/docs/WEBTEXT/29076/64849/S59HND01.htm#:~:text=El%20presente%20C%C3%B3digo%20regula%20las,compensaci%C3%B3n%20equitativa%20de%20su%20inversi%C3%B3n.

    country = "HN"

    def _populate(self, year):
        super()._populate(year)

        # New Year's Day
        self[date(year, JAN, 1)] = "Año Nuevo [New Year's Day]"

        easter_date = easter(year)
        # Maundy Thursday
        self[easter_date + rd(days=-3)] = "Jueves Santo [Maundy Thursday]"

        # Good Friday
        self[easter_date + rd(days=-2)] = "Viernes Santo [Good Friday]"

        # Holy Saturday
        self[easter_date + rd(days=-1)] = "Sábado de Gloria [Holy Saturday]"

        # Panamerican Day
        self[date(year, APR, 14)] = "Día de las Américas [Panamerican Day]"

        # Labor Day
        self[date(year, MAY, 1)] = "Día del Trabajo [Labor Day]"

        # Independence Day
        self[
            date(year, SEP, 15)
        ] = "Día de la Independencia [Independence Day]"

        # https://www.tsc.gob.hn/web/leyes/Decreto_78-2015_Traslado_de_Feriados_Octubre.pdf
        if year <= 2014:
            # Morazan's Day
            self[date(year, OCT, 3)] = "Día de Morazán [Morazan's Day]"

            # Columbus Day
            self[date(year, OCT, 12)] = "Día de la Raza [Columbus Day]"

            # Army Day
            self[date(year, OCT, 21)] = "Día de las Fuerzas Armadas [Army Day]"
        else:
            # Morazan Weekend
            # (First Wednesday of October from 12 noon to Saturday 12 noon)
            holiday_name = "Semana Morazánica [Morazan Weekend]"
            first_wednesday = date(year, OCT, 1) + rd(weekday=WE(+1))
            self[first_wednesday] = holiday_name
            self[first_wednesday + rd(days=1)] = holiday_name
            self[first_wednesday + rd(days=2)] = holiday_name

        # Christmas
        self[date(year, DEC, 25)] = "Navidad [Christmas]"


class HN(Honduras):
    pass


class HND(Honduras):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

import warnings
from datetime import date

from holidays.constants import JAN, MAR, APR, MAY, JUN, AUG, OCT, NOV, DEC
from holidays.holiday_base import HolidayBase


class India(HolidayBase):
    """
    https://www.india.gov.in/calendar
    https://www.india.gov.in/state-and-ut-holiday-calendar
    https://en.wikipedia.org/wiki/Public_holidays_in_India
    https://www.calendarlabs.com/holidays/india/2021
    https://slusi.dacnet.nic.in/watershedatlas/list_of_state_abbreviation.htm
    https://vahan.parivahan.gov.in/vahan4dashboard/
    """

    country = "IN"

    subdivisions = [
        "AN",  # Andaman and Nicobar Islands
        "AP",  # Andhra Pradesh
        "AR",  # Arunachal Pradesh
        "AS",  # Assam
        "BR",  # Bihar
        "CG",  # Chhattisgarh
        "CH",  # Chandigarh
        "DD",  # Daman and Diu
        "DH",  # Dadra and Nagar Haveli
        "DL",  # Delhi
        "GA",  # Goa
        "GJ",  # Gujarat
        "HP",  # Himachal Pradesh
        "HR",  # Haryana
        "JH",  # Jharkhand
        "JK",  # Jammu and Kashmir
        "KA",  # Karnataka
        "KL",  # Kerala
        "LA",  # Ladakh
        "LD",  # Lakshadweep
        "MH",  # Maharashtra
        "ML",  # Meghalaya
        "MN",  # Manipur
        "MP",  # Madhya Pradesh
        "MZ",  # Mizoram
        "NL",  # Nagaland
        "OR",  # Orissa / Odisha (Govt sites (dacnet/vahan) use code "OR")
        "PB",  # Punjab
        "PY",  # Pondicherry
        "RJ",  # Rajasthan
        "SK",  # Sikkim
        "TN",  # Tamil Nadu
        "TR",  # Tripura
        "TS",  # Telangana
        "UK",  # Uttarakhand
        "UP",  # Uttar Pradesh
        "WB",  # West Bengal
    ]

    def _populate(self, year):
        super()._populate(year)

        # Pongal/ Makar Sankranti
        self[date(year, JAN, 14)] = "Makar Sankranti / Pongal"

        if year >= 1950:
            # Republic Day
            self[date(year, JAN, 26)] = "Republic Day"

        if year >= 1947:
            # Independence Day
            self[date(year, AUG, 15)] = "Independence Day"

        # Gandhi Jayanti
        self[date(year, OCT, 2)] = "Gandhi Jayanti"

        # Labour Day
        self[date(year, MAY, 1)] = "Labour Day"

        # Christmas
        self[date(year, DEC, 25)] = "Christmas"

        # GJ: Gujarat
        if self.subdiv == "GJ":
            self[date(year, JAN, 14)] = "Uttarayan"
            self[date(year, MAY, 1)] = "Gujarat Day"
            self[date(year, OCT, 31)] = "Sardar Patel Jayanti"

        if self.subdiv == "BR":
            self[date(year, MAR, 22)] = "Bihar Day"

        if self.subdiv == "RJ":
            self[date(year, MAR, 30)] = "Rajasthan Day"
            self[date(year, JUN, 15)] = "Maharana Pratap Jayanti"

        if self.subdiv == "OR":
            self[date(year, APR, 1)] = "Odisha Day (Utkala Dibasa)"
            self[date(year, APR, 15)] = (
                "Maha Vishuva Sankranti / Pana" " Sankranti"
            )

        if self.subdiv in {
            "OR",
            "AP",
            "BR",
            "WB",
            "KL",
            "HR",
            "MH",
            "UP",
            "UK",
            "TN",
        }:
            self[date(year, APR, 14)] = "Dr. B. R. Ambedkar's Jayanti"

        if self.subdiv == "TN":
            self[date(year, APR, 14)] = "Puthandu (Tamil New Year)"
            self[date(year, APR, 15)] = "Puthandu (Tamil New Year)"

        if self.subdiv == "WB":
            self[date(year, APR, 14)] = "Pohela Boishakh"
            self[date(year, APR, 15)] = "Pohela Boishakh"
            self[date(year, MAY, 9)] = "Rabindra Jayanti"

        if self.subdiv == "AS":
            self[date(year, APR, 15)] = "Bihu (Assamese New Year)"

        if self.subdiv == "MH":
            self[date(year, MAY, 1)] = "Maharashtra Day"
            self[date(year, OCT, 15)] = "Dussehra"

        if self.subdiv == "SK":
            self[date(year, MAY, 16)] = "Annexation Day"

        if self.subdiv == "KA":
            self[date(year, NOV, 1)] = "Karnataka Rajyotsava"

        if self.subdiv == "AP":
            self[date(year, NOV, 1)] = "Andhra Pradesh Foundation Day"

        if self.subdiv == "HR":
            self[date(year, NOV, 1)] = "Haryana Foundation Day"

        if self.subdiv == "MP":
            self[date(year, NOV, 1)] = "Madhya Pradesh Foundation Day"

        if self.subdiv == "KL":
            self[date(year, NOV, 1)] = "Kerala Foundation Day"

        if self.subdiv == "CG":
            self[date(year, NOV, 1)] = "Chhattisgarh Foundation Day"

        if self.subdiv == "TS":
            self[date(year, OCT, 6)] = "Bathukamma Festival"
            self[date(year, APR, 6)] = "Eid al-Fitr"

        # Directly lifted Diwali and Holi dates from FBProphet from:
        # https://github.com/facebook/prophet/blob/main/python/prophet/hdays.py
        # Warnings kept in place so that users are aware

        if year < 2001 or year > 2030:
            warning_msg = (
                "Diwali and Holi holidays available from 2001 to 2030 only"
            )
            warnings.warn(warning_msg, Warning)

        # https://www.timeanddate.com/holidays/india/diwali
        diwali_dates = {
            2001: (NOV, 14),
            2002: (NOV, 4),
            2003: (OCT, 25),
            2004: (NOV, 12),
            2005: (NOV, 1),
            2006: (OCT, 21),
            2007: (NOV, 9),
            2008: (OCT, 28),
            2009: (OCT, 17),
            2010: (NOV, 5),
            2011: (OCT, 26),
            2012: (NOV, 13),
            2013: (NOV, 3),
            2014: (OCT, 23),
            2015: (NOV, 11),
            2016: (OCT, 30),
            2017: (OCT, 19),
            2018: (NOV, 7),
            2019: (OCT, 27),
            2020: (NOV, 14),
            2021: (NOV, 4),
            2022: (OCT, 24),
            2023: (NOV, 12),
            2024: (NOV, 1),
            2025: (OCT, 20),
            2026: (NOV, 8),
            2027: (OCT, 29),
            2028: (OCT, 17),
            2029: (NOV, 5),
            2030: (OCT, 26),
        }

        # https://www.timeanddate.com/holidays/india/holi
        holi_dates = {
            2001: (MAR, 10),
            2002: (MAR, 29),
            2003: (MAR, 18),
            2004: (MAR, 7),
            2005: (MAR, 26),
            2006: (MAR, 15),
            2007: (MAR, 4),
            2008: (MAR, 22),
            2009: (MAR, 11),
            2010: (MAR, 1),
            2011: (MAR, 20),
            2012: (MAR, 8),
            2013: (MAR, 27),
            2014: (MAR, 17),
            2015: (MAR, 6),
            2016: (MAR, 24),
            2017: (MAR, 13),
            2018: (MAR, 2),
            2019: (MAR, 21),
            2020: (MAR, 10),
            2021: (MAR, 29),
            2022: (MAR, 18),
            2023: (MAR, 8),
            2024: (MAR, 25),
            2025: (MAR, 14),
            2026: (MAR, 4),
            2027: (MAR, 22),
            2028: (MAR, 11),
            2029: (MAR, 1),
            2030: (MAR, 20),
        }

        if year in diwali_dates:
            hol_date = date(year, *diwali_dates[year])
            self[hol_date] = "Diwali"
        if year in holi_dates:
            hol_date = date(year, *holi_dates[year])
            self[hol_date] = "Holi"


class IN(India):
    pass


class IND(India):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAY, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.holiday_base import HolidayBase


class Slovakia(HolidayBase):
    """
    https://sk.wikipedia.org/wiki/Sviatok
    https://www.slov-lex.sk/pravne-predpisy/SK/ZZ/1993/241/20181011.html
    """

    country = "SK"
    special_holidays = {
        2018: (
            (OCT, 30, "100. výročie prijatia Deklarácie slovenského národa"),
        )
    }

    def _populate(self, year):
        super()._populate(year)

        self[date(year, JAN, 1)] = "Deň vzniku Slovenskej republiky"
        self[date(year, JAN, 6)] = (
            "Zjavenie Pána (Traja králi a"
            " vianočnýsviatok pravoslávnych"
            " kresťanov)"
        )

        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Veľký piatok"
        self[easter_date + rd(days=+1)] = "Veľkonočný pondelok"

        self[date(year, MAY, 1)] = "Sviatok práce"

        if year >= 1997:
            self[date(year, MAY, 8)] = "Deň víťazstva nad fašizmom"

        self[date(year, JUL, 5)] = "Sviatok svätého Cyrila a svätého Metoda"

        self[date(year, AUG, 29)] = (
            "Výročie Slovenského národného" " povstania"
        )

        self[date(year, SEP, 1)] = "Deň Ústavy Slovenskej republiky"

        self[date(year, SEP, 15)] = "Sedembolestná Panna Mária"

        self[date(year, NOV, 1)] = "Sviatok Všetkých svätých"

        if year >= 2001:
            self[date(year, NOV, 17)] = "Deň boja za slobodu a demokraciu"

        self[date(year, DEC, 24)] = "Štedrý deň"

        self[date(year, DEC, 25)] = "Prvý sviatok vianočný"

        self[date(year, DEC, 26)] = "Druhý sviatok vianočný"


class SK(Slovakia):
    pass


class SVK(Slovakia):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)
#  Copyright: Kateryna Golovanova <kate@kgthreads.com>, 2022

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV
from holidays.constants import DEC, FRI, SUN
from holidays.holiday_base import HolidayBase


class Bolivia(HolidayBase):
    """
    Bolivia holidays.
    See https://en.wikipedia.org/wiki/Public_holidays_in_Bolivia for details.
    """

    country = "BO"
    subdivisions = [
        "B",
        "C",
        "H",
        "L",
        "N",
        "O",
        "P",
        "S",
        "T",
    ]

    def _populate(self, year):
        super()._populate(year)

        # New Year's Day.
        name = "Año Nuevo"
        if year >= 1825:
            self[date(year, JAN, 1)] = name

        if self.observed and date(year, JAN, 1).weekday() == SUN:
            self[date(year, JAN, 1) + rd(days=+1)] = f"{name} (Observed)"

        # Plurinational State Foundation Day.
        if year >= 2010:
            self[
                date(year, JAN, 22)
            ] = "Nacimiento del Estado Plurinacional de Bolivia"

        # Good Friday.
        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Viernes Santo"

        # La Tablada.
        if self.subdiv == "T":
            self[date(year, APR, 15)] = "La Tablada"

        # Carnival in Oruro.
        if self.subdiv == "O":
            self[easter_date + rd(days=-51)] = "Carnaval de Oruro"

        # Carnival Monday (Observed on Tuesday).
        name = "Feriado por Carnaval"
        self[easter_date + rd(days=-48)] = name
        self[easter_date + rd(days=-47)] = f"{name} (Observed)"

        # Labor Day.
        name = "Dia del trabajo"
        self[date(year, MAY, 1)] = name

        if self.observed and date(year, MAY, 1).weekday() == SUN:
            self[date(year, MAY, 1) + rd(days=+1)] = f"{name} (Observed)"

        # Chuquisaca Day.
        if self.subdiv == "H":
            self[date(year, MAY, 25)] = "Día del departamento de Chuquisaca"

        # Corpus Christi.
        self[easter_date + rd(days=+60)] = "Corpus Christi"

        # Andean New Year.
        name = "Año Nuevo Andino"
        if year >= 2010:
            self[date(year, JUN, 21)] = name

        if self.observed and date(year, JUN, 21).weekday() == SUN:
            self[date(year, JUN, 21) + rd(days=+1)] = f"{name} (Observed)"

        # La Paz Day.
        if self.subdiv == "L":
            self[date(year, JUL, 16)] = "Día del departamento de La Paz"

        # Agrarian Reform Day.
        if year >= 1937:
            self[date(year, AUG, 2)] = "Día de la Revolución Agraria"

        # Independence Day.
        name = "Dia de la Patria"
        if year >= 1825:
            self[date(year, AUG, 6)] = name

        if self.observed and date(year, AUG, 6).weekday() > FRI:
            self[date(year, AUG, 6) + rd(days=+1)] = f"{name} (Observed)"

        # Cochabamba Day.
        if self.subdiv == "C":
            self[date(year, SEP, 14)] = "Día del departamento de Cochabamba"

        # Santa Cruz Day.
        if self.subdiv == "S":
            self[date(year, SEP, 24)] = "Día del departamento de Santa Cruz"

        # Pando Day.
        if self.subdiv == "N":
            self[date(year, OCT, 11)] = "Dia del departamento de Pando"

        # All Soul's Day.
        name = "Todos Santos"
        self[date(year, NOV, 2)] = name

        if self.observed and date(year, NOV, 2).weekday() == SUN:
            self[date(year, NOV, 2) + rd(days=+1)] = f"{name} (Observed)"

        # Potosí Day.
        if self.subdiv == "P":
            self[date(year, NOV, 10)] = "Dia del departamento de Potosí"

        # Beni Day.
        if self.subdiv == "B":
            self[date(year, NOV, 18)] = "Dia del departamento de Beni"

        # Christmas Day.
        name = "Navidad"
        self[date(year, DEC, 25)] = name

        if self.observed and date(year, DEC, 25).weekday() == SUN:
            self[date(year, DEC, 25) + rd(days=+1)] = f"{name} (Observed)"


class BO(Bolivia):
    pass


class BOL(Bolivia):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, FEB, APR, MAY, JUN, SEP, OCT, DEC, SUN
from holidays.holiday_base import HolidayBase


class Mozambique(HolidayBase):
    country = "MZ"

    def _populate(self, year):
        if year <= 1974:
            return
        super()._populate(year)

        self[date(year, JAN, 1)] = "Ano novo"
        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Sexta-feira Santa"

        # carnival is the Tuesday before Ash Wednesday
        # which is 40 days before easter excluding sundays
        self[easter_date + rd(days=-47)] = "Carnaval"

        self[date(year, FEB, 3)] = "Dia dos Heróis Moçambicanos"
        self[date(year, APR, 7)] = "Dia da Mulher Moçambicana"
        self[date(year, MAY, 1)] = "Dia Mundial do Trabalho"
        self[date(year, JUN, 25)] = "Dia da Independência Nacional"
        self[date(year, SEP, 7)] = "Dia da Vitória"
        self[date(year, SEP, 25)] = "Dia das Forças Armadas"
        self[date(year, OCT, 4)] = "Dia da Paz e Reconciliação"
        self[date(year, DEC, 25)] = "Dia de Natal e da Família"

        #  whenever a public holiday falls on a Sunday,
        # it rolls over to the following Monday
        if self.observed:
            for k, v in list(self.items()):
                if k.weekday() == SUN and k.year == year:
                    self[k + rd(days=+1)] = v + " (PONTE)"


class MZ(Mozambique):
    pass


class MOZ(Mozambique):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.constants import JAN, MAR, MAY, SEP, OCT, DEC
from holidays.holiday_base import HolidayBase
from holidays.utils import _islamic_to_gre


class Uzbekistan(HolidayBase):
    """
    https://www.officeholidays.com/countries/uzbekistan
    """

    country = "UZ"

    def _populate(self, year):
        super()._populate(year)

        """Populate the holidays for a given year"""
        # New Year's holiday
        self[date(year, JAN, 1)] = "New Year"

        # Women's Day
        self[date(year, MAR, 8)] = "Women's Day"

        # Nauryz Holiday
        self[date(year, MAR, 21)] = "Nauryz"

        # Ramadan Khait
        # Date of observance is announced yearly, This is an estimate.
        for hol_date in _islamic_to_gre(year, 10, 1):
            self[hol_date] = "Ramadan Khait"

        # Memorial Day
        self[date(year, MAY, 9)] = "Memorial Day"

        # Kurban Khait
        # Date of observance is announced yearly, This is an estimate.
        for hol_date in _islamic_to_gre(year, 12, 10):
            self[hol_date] = "Kurban Khait"

            # Independence Day
        self[date(year, SEP, 1)] = "Independence Day"

        # Teacher's Day
        self[date(year, OCT, 1)] = "Teacher's Day"

        # Constitution Day
        self[date(year, DEC, 8)] = "Constitution"


class UZ(Uzbekistan):
    pass


class UZB(Uzbekistan):
    pass



#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import easter
from dateutil.relativedelta import SU
from dateutil.relativedelta import relativedelta as rd

from holidays.constants import JAN, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT
from holidays.constants import NOV, DEC
from holidays.holiday_base import HolidayBase


class Brazil(HolidayBase):
    """
    https://pt.wikipedia.org/wiki/Feriados_no_Brasil
    """

    country = "BR"
    subdivisions = [
        "AC",
        "AL",
        "AP",
        "AM",
        "BA",
        "CE",
        "DF",
        "ES",
        "GO",
        "MA",
        "MT",
        "MS",
        "MG",
        "PA",
        "PB",
        "PE",
        "PI",
        "PR",
        "RJ",
        "RN",
        "RS",
        "RO",
        "RR",
        "SC",
        "SP",
        "SE",
        "TO",
    ]

    def _populate(self, year):
        super()._populate(year)

        # New Year's Day
        self[date(year, JAN, 1)] = "Ano novo"

        self[date(year, APR, 21)] = "Tiradentes"

        self[date(year, MAY, 1)] = "Dia Mundial do Trabalho"

        self[date(year, SEP, 7)] = "Independência do Brasil"

        self[date(year, OCT, 12)] = "Nossa Senhora Aparecida"

        self[date(year, NOV, 2)] = "Finados"

        self[date(year, NOV, 15)] = "Proclamação da República"

        # Christmas Day
        self[date(year, DEC, 25)] = "Natal"

        easter_date = easter(year)
        self[easter_date + rd(days=-2)] = "Sexta-feira Santa"

        self[easter_date] = "Páscoa"

        self[easter_date + rd(days=+60)] = "Corpus Christi"

        quaresma = easter_date + rd(days=-46)
        self[quaresma] = "Quarta-feira de cinzas (Início da Quaresma)"

        self[quaresma + rd(days=-1)] = "Carnaval"

        if self.subdiv == "AC":
            self[date(year, JAN, 23)] = "Dia do evangélico"
            self[date(year, JUN, 15)] = "Aniversário do Acre"
            self[date(year, SEP, 5)] = "Dia da Amazônia"
            self[date(year, NOV, 17)] = (
                "Assinatura do Tratado de" " Petrópolis"
            )

        if self.subdiv == "AL":
            self[date(year, JUN, 24)] = "São João"
            self[date(year, JUN, 29)] = "São Pedro"
            self[date(year, SEP, 16)] = "Emancipação política de Alagoas"
            self[date(year, NOV, 20)] = "Consciência Negra"

        if self.subdiv == "AP":
            self[date(year, MAR, 19)] = "Dia de São José"
            self[date(year, JUL, 25)] = "São Tiago"
            self[date(year, OCT, 5)] = "Criação do estado"
            self[date(year, NOV, 20)] = "Consciência Negra"

        if self.subdiv == "AM":
            self[date(year, SEP, 5)] = (
                "Elevação do Amazonas" " à categoria de província"
            )
            self[date(year, NOV, 20)] = "Consciência Negra"
            self[date(year, DEC, 8)] = "Dia de Nossa Senhora da Conceição"

        if self.subdiv == "BA":
            self[date(year, JUL, 2)] = "Independência da Bahia"

        if self.subdiv == "CE":
            self[date(year, MAR, 19)] = "São José"
            self[date(year, MAR, 25)] = "Data Magna do Ceará"

        if self.subdiv == "DF":
            self[date(year, APR, 21)] = "Fundação de Brasília"
            self[date(year, NOV, 30)] = "Dia do Evangélico"

        if self.subdiv == "ES":
            self[date(year, OCT, 28)] = "Dia do Servidor Público"

        if self.subdiv == "GO":
            self[date(year, OCT, 28)] = "Dia do Servidor Público"

        if self.subdiv == "MA":
            self[date(year, JUL, 28)] = (
                "Adesão do Maranhão" " à independência do Brasil"
            )
            self[date(year, DEC, 8)] = "Dia de Nossa Senhora da Conceição"

        if self.subdiv == "MT":
            self[date(year, NOV, 20)] = "Consciência Negra"

        if self.subdiv == "MS":
            self[date(year, OCT, 11)] = "Criação do estado"

        if self.subdiv == "MG":
            self[date(year, APR, 21)] = "Data Magna de MG"

        if self.subdiv == "PA":
            self[date(year, AUG, 15)] = (
                "Adesão do Grão-Pará" " à independência do Brasil"
            )

        if self.subdiv == "PB":
            self[date(year, AUG, 5)] = "Fundação do Estado"

        if self.subdiv == "PE":
            self[date(year, MAR, 6)] = "Revolução Pernambucana (Data Magna)"
            self[date(year, JUN, 24)] = "São João"

        if self.subdiv == "PI":
            self[date(year, MAR, 13)] = "Dia da Batalha do Jenipapo"
            self[date(year, OCT, 19)] = "Dia do Piauí"

        if self.subdiv == "PR":
            self[date(year, DEC, 19)] = "Emancipação do Paraná"

        if self.subdiv == "RJ":
            self[date(year, APR, 23)] = "Dia de São Jorge"
            self[date(year, OCT, 28)] = "Dia do Funcionário Público"
            self[date(year, NOV, 20)] = "Zumbi dos Palmares"

        if self.subdiv == "RN":
            self[date(year, JUN, 29)] = "Dia de São Pedro"
            self[date(year, OCT, 3)] = "Mártires de Cunhaú e Uruaçuu"

        if self.subdiv == "RS":
            self[date(year, SEP, 20)] = "Revolução Farroupilha"

        if self.subdiv == "RO":
            self[date(year, JAN, 4)] = "Criação do estado"
            self[date(year, JUN, 18)] = "Dia do Evangélico"

        if self.subdiv == "RR":
            self[date(year, OCT, 5)] = "Criação de Roraima"

        if self.subdiv == "SC":
            dt = date(year, AUG, 11)
            if year >= 2018 and not self._is_weekend(dt):
                dt += rd(weekday=SU)
            self[dt] = "Criação da capitania, separando-se de SP"

        if self.subdiv == "SP":
            self[date(year, JUL, 9)] = "Revolução Constitucionalista de 1932"

        if self.subdiv == "SE":
            self[date(year, JUL, 8)] = "Autonomia política de Sergipe"

        if self.subdiv == "TO":
            self[date(year, JAN, 1)] = "Instalação de Tocantins"
            self[date(year, SEP, 8)] = "Nossa Senhora da Natividade"
            self[date(year, OCT, 5)] = "Criação de Tocantins"


class BR(Brazil):
    pass


class BRA(Brazil):
    pass
